@@ -1,29 +1,44 @@
 /* cyradm.c -- Cyrus administrative client
- # Copyright 1998 Carnegie Mellon University
- # 
- # No warranties, either expressed or implied, are made regarding the
- # operation, use, or results of the software.
- #
- # Permission to use, copy, modify and distribute this software and its
- # documentation is hereby granted for non-commercial purposes only
- # provided that this copyright notice appears in all copies and in
- # supporting documentation.
- #
- # Permission is also granted to Internet Service Providers and others
- # entities to use the software for internal purposes.
- #
- # The distribution, modification or sale of a product which uses or is
- # based on the software, in whole or in part, for commercial purposes or
- # benefits requires specific, additional permission from:
- #
- #  Office of Technology Transfer
- #  Carnegie Mellon University
- #  5000 Forbes Avenue
- #  Pittsburgh, PA  15213-3890
- #  (412) 268-4387, fax: (412) 268-7395
- #  tech-transfer@andrew.cmu.edu
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
+#include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
@@ -61,8 +76,7 @@ static int cmd_setquota(), cmd_listquota(), cmd_listquotaroot();
 /*
  * Initialize the cyradm package
  */
-int Cyradm_Init(interp)
-Tcl_Interp *interp;
+int Cyradm_Init(Tcl_Interp *interp)
 {
     Tcl_CreateCommand(interp, "cyradm", Cyradm_CyradmCmd,
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
@@ -215,7 +229,7 @@ int argc;
 char **argv;
 {
     struct admconn *conn = (struct admconn *)clientData;
-    int numcmd = 1, r;
+    int numcmd = 1;
 
     conn->cmd_done = 0;
     conn->cmd_result = TCL_OK;
@@ -446,7 +460,6 @@ static void callback_capability(struct imclient *imclient,
 
 static int cmd_login(struct admconn *conn, char *userid, char *pass, int passlen, int tls_layer, int logindisabled)
 {
-  char **comv;
 
   if (logindisabled==1)
   {
@@ -466,7 +479,7 @@ static int cmd_login(struct admconn *conn, char *userid, char *pass, int passlen
 
 
   imclient_send(conn->imclient, callback_finish, (void *)conn,
-		"LOGIN %s {%d+}\r\n%s", 
+		"LOGIN \"%s\" {%d+}\r\n%s", 
 		userid, passlen, pass);
 
   conn->cmd_done = 0;
@@ -490,7 +503,7 @@ char **argv;
     char *pwcommand = 0;
     char *user = 0;
     char *p;
-    int r;
+    int r = 0;
     int minssf=0;     /* default to allow any security layer */
     int maxssf=10000;
     char *mech = NULL;
@@ -605,7 +618,7 @@ char **argv;
 
 	/* Expand the %-escapes in pwcommand */
 	Tcl_DStringInit(&command);
-	while (p = strchr(pwcommand, '%')) {
+	while ((p = strchr(pwcommand, '%'))!=NULL) {
 	    Tcl_DStringAppend(&command, pwcommand, p - pwcommand);
 	    switch (*++p) {
 	    case '%':
@@ -673,7 +686,7 @@ struct imclient_reply *reply;
 {
     struct mailboxdata *mailboxdata = (struct mailboxdata *)rock;
     char *s, *end;
-    char *mailbox, *attributes, *separator;
+    char *mailbox, *attributes, *separator = NULL;
     int c;
 
     s = reply->text;
@@ -705,7 +718,7 @@ struct imclient_reply *reply;
     Tcl_DStringStartSublist(&mailboxdata->data);
     Tcl_DStringAppendElement(&mailboxdata->data, mailbox);
     Tcl_DStringStartSublist(&mailboxdata->data);
-    for (s = attributes; end = strchr(s, ' '); s = end+1) {
+    for (s = attributes; (end = strchr(s, ' '))!=NULL ; s = end+1) {
 	*s = '\0';
 	Tcl_DStringAppendElement(&mailboxdata->data, s);
     }
@@ -725,8 +738,7 @@ Tcl_Interp *interp;
 int argc;
 char **argv;
 {
-    char *command = argv[0];
-    int i;
+    char *command = argv[0];    
     struct mailboxdata mailboxdata;
     int subscribed = 0;
     char *reference = "";
@@ -793,7 +805,6 @@ char **argv;
 {
     int i, num;
     char *mailbox;
-    char *rights;
 
     if (argc < 4) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -906,7 +917,6 @@ Tcl_Interp *interp;
 int argc;
 char **argv;
 {
-    int i;
     struct acldata acldata;
 
     if (argc != 3) {
