@@ -1,33 +1,50 @@
 /* index.c -- Routines for dealing with the index file in the imapd
  *
- * Copyright 1998 Carnegie Mellon University
- * 
- * No warranties, either expressed or implied, are made regarding the
- * operation, use, or results of the software.
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
- * Permission to use, copy, modify and distribute this software and its
- * documentation is hereby granted for non-commercial purposes only
- * provided that this copyright notice appears in all copies and in
- * supporting documentation.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission is also granted to Internet Service Providers and others
- * entities to use the software for internal purposes.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * The distribution, modification or sale of a product which uses or is
- * based on the software, in whole or in part, for commercial purposes or
- * benefits requires specific, additional permission from:
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- *  Office of Technology Transfer
- *  Carnegie Mellon University
- *  5000 Forbes Avenue
- *  Pittsburgh, PA  15213-3890
- *  (412) 268-4387, fax: (412) 268-7395
- *  tech-transfer@andrew.cmu.edu
  *
  */
 /*
- * $Id: index.c,v 1.91 2000/01/04 19:42:23 leg Exp $
+ * $Id: index.c,v 1.102 2000/05/23 20:52:17 robeson Exp $
  */
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
@@ -38,6 +55,7 @@
 #include <syslog.h>
 #include <com_err.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include "acl.h"
 #include "util.h"
@@ -51,9 +69,12 @@
 #include "append.h"
 #include "charset.h"
 #include "xmalloc.h"
+#include "seen.h"
 
 extern int errno;
 
+extern void printastring (const char *s);
+
 /* The index and cache files, mapped into memory */
 static const char *index_base;
 static unsigned long index_len;
@@ -351,15 +372,16 @@ int checkseen;
 
     /* Check Flags */
     if (checkseen) index_checkseen(mailbox, 0, usinguid, oldexists);
+    else if (oldexists == -1) seen_unlock(seendb);
     for (i = 1; i <= imapd_exists && seenflag[i]; i++);
     if (i == imapd_exists + 1) allseen = mailbox->last_uid;
     if (oldexists == -1) {
 	if (imapd_exists && i <= imapd_exists) {
-	    prot_printf(imapd_out, "* OK [UNSEEN %u] \r\n", i);
+	    prot_printf(imapd_out, "* OK [UNSEEN %u]  \r\n", i);
 	}
-        prot_printf(imapd_out, "* OK [UIDVALIDITY %u] \r\n",
+        prot_printf(imapd_out, "* OK [UIDVALIDITY %u]  \r\n",
 		    mailbox->uidvalidity);
-	prot_printf(imapd_out, "* OK [UIDNEXT %u] \r\n",
+	prot_printf(imapd_out, "* OK [UIDNEXT %u]  \r\n",
 		    mailbox->last_uid + 1);
     }
 
@@ -423,8 +445,8 @@ int oldexists;
      */
     old = seenuids;
     new = newseenuids;
-    while (isdigit(*old)) oldnext = oldnext * 10 + *old++ - '0';
-    while (isdigit(*new)) newnext = newnext * 10 + *new++ - '0';
+    while (isdigit((int) *old)) oldnext = oldnext * 10 + *old++ - '0';
+    while (isdigit((int) *new)) newnext = newnext * 10 + *new++ - '0';
 
     for (msgno = 1; msgno <= imapd_exists; msgno++) {
 	uid = UID(msgno);
@@ -438,7 +460,7 @@ int oldexists;
 		oldnext = 0;
 		if (!*old) oldnext = mailbox->last_uid+1;
 		else old++;
-		while (isdigit(*old)) oldnext = oldnext * 10 + *old++ - '0';
+		while (isdigit((int) *old)) oldnext = oldnext * 10 + *old++ - '0';
 		oldnext += oldseen;
 	    }
 	}
@@ -455,7 +477,7 @@ int oldexists;
 		    neweof++;
 		}
 		else new++;
-		while (isdigit(*new)) newnext = newnext * 10 + *new++ - '0';
+		while (isdigit((int) *new)) newnext = newnext * 10 + *new++ - '0';
 		newnext += newseen;
 	    }
 	}
@@ -502,11 +524,13 @@ int oldexists;
 	    for (msgno = 1; msgno <= imapd_exists; msgno++) {
 		if (!seenflag[msgno]) break;
 	    }
+#if TOIMSP
 	    if (msgno == imapd_exists + 1) {
 		toimsp(mailbox->name, mailbox->uidvalidity,
 		       "SEENsnn", imapd_userid, mailbox->last_uid,
 		       seen_last_change, 0);
 	    }
+#endif
 	}
 	return;
     }
@@ -567,7 +591,7 @@ int oldexists;
 		neweof++;
 	    }
 	    else new++;
-	    while (isdigit(*new)) newnext = newnext * 10 + *new++ - '0';
+	    while (isdigit((int) *new)) newnext = newnext * 10 + *new++ - '0';
 	    newnext += newseen;
 	}
     }
@@ -585,7 +609,7 @@ int oldexists;
 		/* There's a ":M" after the ",N".  Parse/include that too. */
 		new++;
 		newnext = 0;
-		while (isdigit(*new)) newnext = newnext * 10 + *new++ - '0';
+		while (isdigit((int) *new)) newnext = newnext * 10 + *new++ - '0';
 	    }
 	    uid = newnext;
 	    newseen++;		/* Forget we parsed ",N" */
@@ -651,6 +675,7 @@ int oldexists;
 	return;
     }
 
+#if TOIMSP
     if (newallseen) {
 	toimsp(mailbox->name, mailbox->uidvalidity, "SEENsnn", imapd_userid,
 	       mailbox->last_uid, seen_last_change, 0);
@@ -659,7 +684,8 @@ int oldexists;
 	toimsp(mailbox->name, mailbox->uidvalidity, "SEENsnn", imapd_userid,
 	       0, seen_last_change, 0);
     }
-    
+#endif    
+
     free(newseenuids);
     seenuids = saveseenuids;
 }
@@ -824,6 +850,14 @@ int nflags;
     r = index_forsequence(mailbox, sequence, usinguid,
 			  index_storeflag, (char *)storeargs, NULL);
 
+
+    if (mailbox->dirty==1)
+    {
+	/* xxx what to do on failure? */
+	mailbox_write_index_header(mailbox);
+	mailbox->dirty = 0;
+    }
+
     mailbox_unlock_index(mailbox);
 
     /* Refresh the index file, for systems without mmap() */
@@ -867,21 +901,22 @@ int usinguid;
  * Performs a COPY command
  */
 int
-index_copy(mailbox, sequence, usinguid, name, copyuidp)
-struct mailbox *mailbox;
-char *sequence;
-int usinguid;
-char *name;
-char **copyuidp;
+index_copy(struct mailbox *mailbox, 
+	   char *sequence, 
+	   int usinguid,
+	   char *name, 
+	   char **copyuidp)
 {
     static struct copyargs copyargs;
     int i;
     unsigned long totalsize = 0;
     int r;
-    struct mailbox append_mailbox;
+    struct appendstate append_mailbox;
     char *copyuid;
     int copyuid_len, copyuid_size;
     int sepchar;
+    unsigned long uidvalidity;
+    unsigned long startuid, num;
 
     copyargs.nummsg = 0;
     index_forsequence(mailbox, sequence, usinguid, index_copysetup,
@@ -902,11 +937,12 @@ char **copyuidp;
 
     r = append_copy(mailbox, &append_mailbox, copyargs.nummsg,
 		    copyargs.copymsg, imapd_userid);
-
+    if (!r) append_commit(&append_mailbox, &uidvalidity, &startuid,
+			  &num);
     if (!r) {
 	copyuid_size = 1024;
 	copyuid = xmalloc(copyuid_size);
-	sprintf(copyuid, "%u", append_mailbox.uidvalidity);
+	sprintf(copyuid, "%lu", uidvalidity);
 	copyuid_len = strlen(copyuid);
 	sepchar = ' ';
 
@@ -915,7 +951,7 @@ char **copyuidp;
 		copyuid_size += 1024;
 		copyuid = xrealloc(copyuid, copyuid_size);
 	    }
-	    sprintf(copyuid+copyuid_len, "%c%u", sepchar,
+	    sprintf(copyuid+copyuid_len, "%c%lu", sepchar,
 		    copyargs.copymsg[i].uid);
 	    copyuid_len += strlen(copyuid+copyuid_len);
 	    if (i+1 < copyargs.nummsg &&
@@ -924,25 +960,21 @@ char **copyuidp;
 		    i++;
 		} while (i+1 < copyargs.nummsg &&
 			 copyargs.copymsg[i+1].uid == copyargs.copymsg[i].uid + 1);
-		sprintf(copyuid+copyuid_len, ":%u",
+		sprintf(copyuid+copyuid_len, ":%lu",
 			copyargs.copymsg[i].uid);
 		copyuid_len += strlen(copyuid+copyuid_len);
 	    }
 	    sepchar = ',';
 	}
-	if (copyargs.nummsg == 1) {
-	    sprintf(copyuid+copyuid_len, " %u", append_mailbox.last_uid);
-	}
-	else {
-	    sprintf(copyuid+copyuid_len, " %u:%u",
-		    append_mailbox.last_uid - copyargs.nummsg + 1,
-		    append_mailbox.last_uid);
+	if (num == 1) {
+	    sprintf(copyuid+copyuid_len, " %lu", startuid);
+	} else {
+	    sprintf(copyuid+copyuid_len, " %lu:%lu",
+		    startuid, startuid + num - 1);
 	}
 	*copyuidp = copyuid;
     }
 
-    mailbox_close(&append_mailbox);
-
     return r;
 }
 
@@ -1076,9 +1108,7 @@ unsigned lowuid;
 int
 index_getstate(mailbox)
 struct mailbox *mailbox;
-{
-    int r;
-    int msgno;
+{    
 
     prot_printf(imapd_out, "* XSTATE %u %u\r\n", mailbox->index_mtime,
 		seen_last_change);
@@ -1221,7 +1251,7 @@ index_forsequence(struct mailbox* mailbox,
     }
 
     for (;;) {
-	if (isdigit(*sequence)) {
+	if (isdigit((int) *sequence)) {
 	    start = start*10 + *sequence - '0';
 	}
 	else if (*sequence == '*') {
@@ -1230,7 +1260,7 @@ index_forsequence(struct mailbox* mailbox,
 	else if (*sequence == ':') {
 	    end = 0;
 	    sequence++;
-	    while (isdigit(*sequence)) {
+	    while (isdigit((int) *sequence)) {
 		end = end*10 + *sequence++ - '0';
 	    }
 	    if (*sequence == '*') {
@@ -1288,7 +1318,7 @@ int usinguid;
     unsigned i, start = 0, end;
 
     for (;;) {
-	if (isdigit(*sequence)) {
+	if (isdigit((int) *sequence)) {
 	    start = start*10 + *sequence - '0';
 	}
 	else if (*sequence == '*') {
@@ -1298,7 +1328,7 @@ int usinguid;
 	else if (*sequence == ':') {
 	    end = 0;
 	    sequence++;
-	    while (isdigit(*sequence)) {
+	    while (isdigit((int) *sequence)) {
 		end = end*10 + *sequence++ - '0';
 	    }
 	    if (*sequence == '*') {
@@ -1373,6 +1403,7 @@ unsigned octet_count;
 	    line = msg_base + offset;
 	    p = memchr(line, '\n', msg_size - offset);
 	    if (!p) {
+		/* partial last line */
 		p = msg_base + msg_size;
 		offset--;	/* hack to keep from going off end
 				 * of mapped region on next iteration */
@@ -1443,7 +1474,7 @@ unsigned start_octet;
 unsigned octet_count;
 {
     char *p;
-    int skip;
+    int skip = 0;
     int fetchmime = 0;
 
     cacheitem += 4;
@@ -1455,9 +1486,9 @@ unsigned octet_count;
 	if (*p == '<') {
 	    p++;
 	    start_octet = octet_count = 0;
-	    while (isdigit(*p)) start_octet = start_octet * 10 + *p++ - '0';
+	    while (isdigit((int) *p)) start_octet = start_octet * 10 + *p++ - '0';
 	    p++;			/* Skip over '.' */
-	    while (isdigit(*p)) octet_count = octet_count * 10 + *p++ - '0';
+	    while (isdigit((int) *p)) octet_count = octet_count * 10 + *p++ - '0';
 	    start_octet++;	/* Make 1-based */
 	}
 
@@ -1468,7 +1499,7 @@ unsigned octet_count;
 
     while (*p != ']' && *p != 'M') {
 	skip = 0;
-	while (isdigit(*p)) skip = skip * 10 + *p++ - '0';
+	while (isdigit((int) *p)) skip = skip * 10 + *p++ - '0';
 	if (*p == '.') p++;
 
 	/* section number too large */
@@ -1516,9 +1547,9 @@ unsigned octet_count;
     if (*p == '<') {
 	p++;
 	start_octet = octet_count = 0;
-	while (isdigit(*p)) start_octet = start_octet * 10 + *p++ - '0';
+	while (isdigit((int) *p)) start_octet = start_octet * 10 + *p++ - '0';
 	p++;			/* Skip over '.' */
-	while (isdigit(*p)) octet_count = octet_count * 10 + *p++ - '0';
+	while (isdigit((int) *p)) octet_count = octet_count * 10 + *p++ - '0';
 	start_octet++;		/* Make 1-based */
     }
 
@@ -1563,7 +1594,7 @@ const char *cacheitem;
 
     while (*p != 'H') {
 	skip = 0;
-	while (isdigit(*p)) skip = skip * 10 + *p++ - '0';
+	while (isdigit((int) *p)) skip = skip * 10 + *p++ - '0';
 	if (*p == '.') p++;
 
 	/* section number too large */
@@ -1592,9 +1623,9 @@ const char *cacheitem;
     if (p[1] == '<') {
 	p += 2;
 	start_octet = octet_count = 0;
-	while (isdigit(*p)) start_octet = start_octet * 10 + *p++ - '0';
+	while (isdigit((int) *p)) start_octet = start_octet * 10 + *p++ - '0';
 	p++;			/* Skip over '.' */
-	while (isdigit(*p)) octet_count = octet_count * 10 + *p++ - '0';
+	while (isdigit((int) *p)) octet_count = octet_count * 10 + *p++ - '0';
 	start_octet++;		/* Make 1-based */
     }
 
@@ -1689,7 +1720,7 @@ unsigned size;
     if (format == MAILBOX_FORMAT_NETNEWS) {
 	left = size;
 	p = buf;
-	while (endline = memchr(msg_base, '\n', left)) {
+	while ((endline = memchr(msg_base, '\n', left))!=NULL) {
 	    linelen = endline - msg_base;
 	    memcpy(p, msg_base, linelen);
 	    p += linelen;
@@ -1843,9 +1874,9 @@ char *trail;
     if (trail[1]) {
 	/* Deal with ]<start.count> */
 	trail += 2;
-	while (isdigit(*trail)) start_octet = start_octet * 10 + *trail++ - '0';
+	while (isdigit((int) *trail)) start_octet = start_octet * 10 + *trail++ - '0';
 	trail++;			/* Skip over '.' */
-	while (isdigit(*trail)) octet_count = octet_count * 10 + *trail++ - '0';
+	while (isdigit((int) *trail)) octet_count = octet_count * 10 + *trail++ - '0';
 
 	if (size <= start_octet) {
 	    crlf_start = start_octet - size;
@@ -1922,7 +1953,7 @@ struct mailbox *mailbox;
 	}
     }
     if (sepchar == '(') prot_printf(imapd_out, "(");
-    prot_printf(imapd_out, ")] \r\n");
+    prot_printf(imapd_out, ")]  \r\n");
 }
 
 /*
@@ -1940,7 +1971,7 @@ time_t last_updated;
 {
     int sepchar = '(';
     unsigned flag;
-    bit32 flagmask;
+    bit32 flagmask = 0;
 
     for (flag = 0; flag < MAX_USER_FLAGS; flag++) {
 	if ((flag & 31) == 0) {
@@ -2007,7 +2038,6 @@ void *rock;
     int fetchitems = fetchargs->fetchitems;
     const char *msg_base = 0;
     unsigned long msg_size = 0;
-    struct stat sbuf;
     int sepchar;
     int i;
     bit32 user_flags[MAX_USER_FLAGS/32];
@@ -2069,7 +2099,7 @@ void *rock;
 	    gmtnegative = 1;
 	}
 	gmtoff /= 60;
-	sprintf(datebuf, "%2u-%s-%u %.2u:%.2u:%.2u %c%.2u%.2u",
+	sprintf(datebuf, "%2u-%s-%u %.2u:%.2u:%.2u %c%.2lu%.2lu",
 		tm->tm_mday, monthname[tm->tm_mon], tm->tm_year+1900,
 		tm->tm_hour, tm->tm_min, tm->tm_sec,
 		gmtnegative ? '-' : '+', gmtoff/60, gmtoff%60);
@@ -2246,10 +2276,11 @@ void *rock;
     struct index_record record;
     int uid = UID(msgno);
     int low=1, high=mailbox->exists;
-    int mid;
+    int mid = 0;
     int r;
     int firsttry = 1;
     int dirty = 0;
+    bit32 oldflags;
 
     /* Change \Seen flag */
     if (storeargs->operation == STORE_REPLACE && (mailbox->myrights&ACL_SEEN))
@@ -2303,6 +2334,9 @@ void *rock;
 	}
     }
 
+    /* save old for acapmbox foo */
+    oldflags = record.system_flags;
+
     if (storeargs->operation == STORE_REPLACE) {
 	if (!(mailbox->myrights & ACL_WRITE)) {
 	    record.system_flags = (record.system_flags&~FLAG_DELETED) |
@@ -2324,6 +2358,7 @@ void *rock;
     }
     else if (storeargs->operation == STORE_ADD) {
 	if (~record.system_flags & storeargs->system_flags) dirty++;
+
 	record.system_flags |= storeargs->system_flags;
 	for (i = 0; i < MAX_USER_FLAGS/32; i++) {
 	    if (~record.user_flags[i] & storeargs->user_flags[i]) dirty++;
@@ -2332,6 +2367,8 @@ void *rock;
     }
     else {			/* STORE_REMOVE */
 	if (record.system_flags & storeargs->system_flags) dirty++;
+
+	/* change the individual entry */
 	record.system_flags &= ~storeargs->system_flags;
 	for (i = 0; i < MAX_USER_FLAGS/32; i++) {
 	    if (record.user_flags[i] & storeargs->user_flags[i]) dirty++;
@@ -2340,6 +2377,27 @@ void *rock;
     }
 
     if (dirty) {
+	/* update totals */
+	if ( (record.system_flags & FLAG_DELETED) && !(oldflags & FLAG_DELETED))
+	    mailbox->deleted++;
+	if ( !(record.system_flags & FLAG_DELETED) && (oldflags & FLAG_DELETED))
+	    mailbox->deleted--;
+
+	if ( (record.system_flags & FLAG_ANSWERED) && !(oldflags & FLAG_ANSWERED))
+	    mailbox->answered++;
+	if ( !(record.system_flags & FLAG_ANSWERED) && (oldflags & FLAG_ANSWERED))
+	    mailbox->answered--;
+
+	if ( (record.system_flags & FLAG_FLAGGED) && !(oldflags & FLAG_FLAGGED))
+	    mailbox->flagged++;
+	if ( !(record.system_flags & FLAG_FLAGGED) && (oldflags & FLAG_FLAGGED))
+	    mailbox->flagged--;
+
+	/* either a system or user flag changed. need to at least touch acap
+	   to change the modtime */
+	mailbox->dirty = 1;
+	
+
 	/* If .SILENT, assume client has updated their cache */
 	if (storeargs->silent && flagreport[msgno] &&
 	    flagreport[msgno] == record.last_updated) {
@@ -2387,7 +2445,6 @@ struct mapfile *msgfile;
     const char *cacheitem;
     int cachelen;
     struct searchsub *s;
-    struct stat sbuf;
 
     if ((searchargs->flags & SEARCH_RECENT_SET) && msgno <= lastnotrecent) return 0;
     if ((searchargs->flags & SEARCH_RECENT_UNSET) && msgno > lastnotrecent) return 0;
@@ -2620,7 +2677,7 @@ char *name;
 char *substr;
 comp_pat *pat;
 {
-    char *p, *q;
+    char *q;
     static struct strlist header;
     static char *buf;
     static int bufsize;
@@ -2649,7 +2706,7 @@ comp_pat *pat;
     index_pruneheader(buf, &header, 0);
     if (!*buf) return 0;	/* Header not present, fail */
     if (!*substr) return 1;	/* Only checking existence, succeed */
-    p = charset_decode1522(buf, NULL, 0);
+    q = charset_decode1522(buf, NULL, 0);
     r = charset_searchstring(substr, pat, q, strlen(q));
     free(q);
     return r;
@@ -2668,7 +2725,7 @@ void *rock;
     struct copyargs *copyargs = (struct copyargs *)rock;
     int flag = 0;
     unsigned userflag;
-    bit32 flagmask;
+    bit32 flagmask = 0;
 
     if (copyargs->nummsg == copyargs->msgalloc) {
 	copyargs->msgalloc += COPYARGSGROW;
