@@ -1,32 +1,50 @@
 /* mailbox.c -- Mailbox manipulation routines
- $Id: mailbox.c,v 1.81 1999/10/02 00:43:04 leg Exp $
+ $Id: mailbox.c,v 1.98 2000/05/23 20:52:20 robeson Exp $
  
- # Copyright 1998 Carnegie Mellon University
- # 
- # No warranties, either expressed or implied, are made regarding the
- # operation, use, or results of the software.
- #
- # Permission to use, copy, modify and distribute this software and its
- # documentation is hereby granted for non-commercial purposes only
- # provided that this copyright notice appears in all copies and in
- # supporting documentation.
- #
- # Permission is also granted to Internet Service Providers and others
- # entities to use the software for internal purposes.
- #
- # The distribution, modification or sale of a product which uses or is
- # based on the software, in whole or in part, for commercial purposes or
- # benefits requires specific, additional permission from:
- #
- #  Office of Technology Transfer
- #  Carnegie Mellon University
- #  5000 Forbes Avenue
- #  Pittsburgh, PA  15213-3890
- #  (412) 268-4387, fax: (412) 268-7395
- #  tech-transfer@andrew.cmu.edu
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+
  *
  */
 
+#include <config.h>
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -38,11 +56,14 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <netinet/in.h>
+#include <sys/un.h>
+#include <sys/socket.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <ctype.h>
 #include <time.h>
 
-#if HAVE_DIRENT_H
+#ifdef HAVE_DIRENT_H
 # include <dirent.h>
 # define NAMLEN(dirent) strlen((dirent)->d_name)
 #else
@@ -60,7 +81,7 @@
 #endif
 
 #include "assert.h"
-#include "config.h"
+#include "imapconf.h"
 #include "acl.h"
 #include "map.h"
 #include "retry.h"
@@ -70,10 +91,16 @@
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
+#include "mboxlist.h"
+#include "acapmbox.h"
+#include "seen.h"
+#include "acappush.h"
 
 static int mailbox_doing_reconstruct = 0;
 static struct mailbox zeromailbox = {-1, -1, -1};
 
+static int mailbox_calculate_flagcounts(struct mailbox *mailbox);
+
 /*
  * Names of the headers we cache in the cyrus.cache file.
  * Any changes to this list require corresponding changes to
@@ -90,20 +117,83 @@ char *mailbox_cache_header_name[] = {
 int mailbox_num_cache_header =
   sizeof(mailbox_cache_header_name)/sizeof(char *);
 
+/* acappush variables */
+static int acappush_sock = -1;
+static struct sockaddr_un acappush_remote;
+static int acappush_remote_len = 0;
+
+/*
+ * Create connection to acappush
+ */
+int mailbox_initialize(void)
+{
+    int s;
+    int fdflags;
+
+    /* if not configured to do acap do nothing */
+    if (config_getstring("acap_server", NULL)==NULL) return 0;
+
+    if ((s = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1) {
+	return IMAP_IOERROR;
+    }
+
+    acappush_remote.sun_family = AF_UNIX;
+    strcpy(acappush_remote.sun_path, config_dir);
+    strcat(acappush_remote.sun_path, FNAME_ACAPPUSH_SOCK);
+    acappush_remote_len = strlen(acappush_remote.sun_path) + 
+	sizeof(acappush_remote.sun_family);
+
+    /* put us in non-blocking mode */
+    fdflags = fcntl(s, F_GETFD, 0);
+    if (fdflags != -1) fdflags = fcntl(s, F_SETFL, O_NONBLOCK | fdflags);
+    if (fdflags == -1) { close(s); return IMAP_IOERROR; }
+
+    acappush_sock = s;
+
+    return 0;
+}
+
+/* create the unique identifier for a mailbox named 'name' with
+ * uidvalidity 'uidvalidity'.  'uniqueid' should be at least 17 bytes
+ * long.  the unique identifier is just the mailbox name hashed to 32
+ * bits followed by the uid, both converted to hex. 
+ */
+#define PRIME (2147484043UL)
+
+void mailbox_make_uniqueid(char *name, unsigned long uidvalidity,
+			   char *uniqueid)
+{
+    unsigned long hash = 0;
+
+    while (*name) {
+	hash *= 251;
+	hash += *name++;
+	hash %= PRIME;
+    }
+    sprintf(uniqueid, "%08lx%08lx", hash, uidvalidity);
+}
+
+/*
+ * Calculate relative filename for the message with UID 'uid'
+ * in 'mailbox'.
+ */
+void mailbox_message_get_fname(struct mailbox *mailbox, unsigned long uid,
+			       char *out)
+{
+    assert(mailbox->format != MAILBOX_FORMAT_NETNEWS);
+
+    sprintf(out, "%lu.", uid);
+}
+
 /*
  * Calculate relative filename for the message with UID 'uid'
  * in 'mailbox'. Returns pointer to static buffer.
  */
-char *mailbox_message_fname(mailbox, uid)
-struct mailbox *mailbox;
-unsigned long uid;
+char *mailbox_message_fname(struct mailbox *mailbox, unsigned long uid)
 {
     static char buf[256];
-    char *p = buf;
-    
-    assert(mailbox->format != MAILBOX_FORMAT_NETNEWS);
 
-    sprintf(p, "%lu.", uid);
+    mailbox_message_get_fname(mailbox, uid, buf);
     return buf;
 }
 
@@ -111,16 +201,14 @@ unsigned long uid;
  * Maps in the content for the message with UID 'uid' in 'mailbox'.
  * Returns map in 'basep' and 'lenp'
  */
-int
-mailbox_map_message(mailbox, iscurrentdir, uid, basep, lenp)
-struct mailbox *mailbox;
-int iscurrentdir;
-unsigned long uid;
-const char **basep;
-unsigned long *lenp;
+int mailbox_map_message(struct mailbox *mailbox,
+			int iscurrentdir,
+			unsigned long uid,
+			const char **basep,
+			unsigned long *lenp)
 {
     int msgfd;
-    static char buf[4096];
+    char buf[4096];
     char *p = buf;
     struct stat sbuf;
     static int newsprefixlen = -1;
@@ -196,16 +284,14 @@ mailbox_reconstructmode()
  * Open and read the header of the mailbox with name 'name'
  * The structure pointed to by 'mailbox' is initialized.
  */
-int
-mailbox_open_header(name, auth_state, mailbox)
-const char *name;
-struct auth_state *auth_state;
-struct mailbox *mailbox;
+int mailbox_open_header(const char *name, 
+			struct auth_state *auth_state,
+			struct mailbox *mailbox)
 {
     char *path, *acl;
     int r;
 
-    r = mboxlist_lookup(name, &path, &acl);
+    r = mboxlist_lookup(name, &path, &acl, NULL);
     if (r) return r;
 
     return mailbox_open_header_path(name, path, acl, auth_state, mailbox, 0);
@@ -216,14 +302,12 @@ struct mailbox *mailbox;
  * path 'path', and ACL 'acl'.
  * The structure pointed to by 'mailbox' is initialized.
  */
-int
-mailbox_open_header_path (name, path, acl, auth_state, mailbox, suppresslog)
-const char *name;
-const char *path;
-const char *acl;
-struct auth_state *auth_state;
-struct mailbox *mailbox;
-int suppresslog;
+int mailbox_open_header_path(const char *name,
+			     const char *path,
+			     const char *acl,
+			     struct auth_state *auth_state,
+			     struct mailbox *mailbox,
+			     int suppresslog)
 {
     char fnamebuf[MAX_MAILBOX_PATH];
     struct stat sbuf;
@@ -275,9 +359,7 @@ int suppresslog;
  * Open the index and cache files for 'mailbox'.  Also 
  * read the index header.
  */
-int 
-mailbox_open_index(mailbox)
-struct mailbox *mailbox;
+int mailbox_open_index(struct mailbox *mailbox)
 {
     char fnamebuf[MAX_MAILBOX_PATH];
     bit32 index_gen = 0, cache_gen = 0;
@@ -346,9 +428,7 @@ struct mailbox *mailbox;
 /*
  * Close the mailbox 'mailbox', freeing all associated resources.
  */
-void
-mailbox_close(mailbox)
-struct mailbox *mailbox;
+void mailbox_close(struct mailbox *mailbox)
 {
     int flag;
 
@@ -372,6 +452,7 @@ struct mailbox *mailbox;
     free(mailbox->name);
     free(mailbox->path);
     free(mailbox->acl);
+    free(mailbox->uniqueid);
     if (mailbox->quota.root) free(mailbox->quota.root);
 
     for (flag = 0; flag < MAX_USER_FLAGS; flag++) {
@@ -385,12 +466,11 @@ struct mailbox *mailbox;
 /*
  * Read the header of 'mailbox'
  */
-int
-mailbox_read_header(mailbox)
-struct mailbox *mailbox;
+int mailbox_read_header(struct mailbox *mailbox)
 {
     int flag;
-    const char *name, *p, *eol;
+    const char *name, *p, *tab, *eol;
+    int oldformat = 0;
 
     /* Check magic number */
     if (mailbox->header_len < sizeof(MAILBOX_HEADER_MAGIC)-1 ||
@@ -401,13 +481,21 @@ struct mailbox *mailbox;
 
     /* Read quota file pathname */
     p = mailbox->header_base + sizeof(MAILBOX_HEADER_MAGIC)-1;
+    tab = memchr(p, '\t', mailbox->header_len - (p - mailbox->header_base));
     eol = memchr(p, '\n', mailbox->header_len - (p - mailbox->header_base));
-    if (!eol) {
-	return IMAP_MAILBOX_BADFORMAT;
+    if (!tab || tab > eol || !eol) {
+	oldformat = 1;
+	if (!eol) return IMAP_MAILBOX_BADFORMAT;
+	else {
+	    syslog(LOG_DEBUG, "mailbox '%s' has old cyrus.header",
+		   mailbox->name);
+	}
+	tab = eol;
     }
     if (mailbox->quota.root) {
-	if (strlen(mailbox->quota.root) != eol-p ||
-	    strncmp(mailbox->quota.root, p, eol-p) != 0) {
+	/* check if this is the same as what's there */
+	if (strlen(mailbox->quota.root) != tab-p ||
+	    strncmp(mailbox->quota.root, p, tab-p) != 0) {
 	    assert(mailbox->quota.lock_count == 0);
 	    if (mailbox->quota.fd != -1) {
 		close(mailbox->quota.fd);
@@ -416,11 +504,20 @@ struct mailbox *mailbox;
 	}
 	free(mailbox->quota.root);
     }
-    if (p < eol) {
-	mailbox->quota.root = xstrndup(p, eol - p);
+    if (p < tab) {
+	mailbox->quota.root = xstrndup(p, tab - p);
+    } else {
+	mailbox->quota.root = NULL;
     }
-    else {
-	mailbox->quota.root = 0;
+
+    if (!oldformat) {
+	/* read uniqueid */
+	p = tab + 1;
+	if (p == eol) return IMAP_MAILBOX_BADFORMAT;
+	mailbox->uniqueid = xstrndup(p, eol - p);
+    } else {
+	/* uniqueid needs to be generated when we know the uidvalidity */
+	mailbox->uniqueid = NULL;
     }
 
     /* Read names of user flags */
@@ -449,15 +546,25 @@ struct mailbox *mailbox;
 	mailbox->flagname[flag++] = NULL;
     }
 
+    if (!mailbox->uniqueid) {
+	char buf[32];
+
+	/* generate uniqueid */
+	mailbox_lock_header(mailbox);
+	mailbox_open_index(mailbox);
+	mailbox_make_uniqueid(mailbox->name, mailbox->uidvalidity, buf);
+	mailbox->uniqueid = xstrdup(buf);
+	mailbox_write_header(mailbox);
+	mailbox_unlock_header(mailbox);
+    }
+
     return 0;
 }
 
 /*
  * Read the acl out of the header of 'mailbox'
  */
-int
-mailbox_read_header_acl(mailbox)
-struct mailbox *mailbox;
+int mailbox_read_header_acl(struct mailbox *mailbox)
 {
     const char *p, *eol;
 
@@ -498,15 +605,13 @@ struct mailbox *mailbox;
 /*
  * Read the the ACL for 'mailbox'.
  */
-int 
-mailbox_read_acl(mailbox, auth_state)
-struct mailbox *mailbox;
-struct auth_state *auth_state;
+int mailbox_read_acl(struct mailbox *mailbox,
+		     struct auth_state *auth_state)
 {
     int r;
     char *acl;
 
-    r = mboxlist_lookup(mailbox->name, (char **)0, &acl);
+    r = mboxlist_lookup(mailbox->name, (char **)0, &acl, NULL);
     if (r) return r;
 
     free(mailbox->acl);
@@ -519,9 +624,7 @@ struct auth_state *auth_state;
 /*
  * Read the header of the index file for mailbox
  */
-int 
-mailbox_read_index_header(mailbox)
-struct mailbox *mailbox;
+int mailbox_read_index_header(struct mailbox *mailbox)
 {
     struct stat sbuf;
 
@@ -577,6 +680,23 @@ struct mailbox *mailbox;
 	    ntohl(*((bit32 *)(mailbox->index_base+OFFSET_UIDVALIDITY)));
     }
 
+    if (mailbox->start_offset < OFFSET_FLAGGED+sizeof(bit32)) {
+	/* calculate them now */
+	if (mailbox_calculate_flagcounts(mailbox))
+	    return IMAP_IOERROR;
+	
+	/* things might have been changed out from under us. reread */
+	mailbox_open_index(mailbox); 
+    } else {
+	mailbox->deleted = 
+	    ntohl(*((bit32 *)(mailbox->index_base+OFFSET_DELETED)));
+	mailbox->answered = 
+	    ntohl(*((bit32 *)(mailbox->index_base+OFFSET_ANSWERED)));
+	mailbox->flagged = 
+	    ntohl(*((bit32 *)(mailbox->index_base+OFFSET_FLAGGED)));
+	mailbox->dirty = 0;
+    }
+
     if (!mailbox_doing_reconstruct &&
 	(mailbox->minor_version < MAILBOX_MINOR_VERSION)) {
 	return IMAP_MAILBOX_BADFORMAT;
@@ -758,7 +878,7 @@ struct mailbox *mailbox;
 
 	if (sbuffd.st_ino == sbuffile.st_ino) break;
 
-	if (r = mailbox_open_index(mailbox)) {
+	if ((r = mailbox_open_index(mailbox))) {
 	    return r;
 	}
     }
@@ -843,9 +963,7 @@ struct quota *quota;
 /*
  * Release lock on the header for 'mailbox'
  */
-void
-mailbox_unlock_header(mailbox)
-struct mailbox *mailbox;
+void mailbox_unlock_header(struct mailbox *mailbox)
 {
     assert(mailbox->header_lock_count != 0);
 
@@ -899,9 +1017,7 @@ struct quota *quota;
 /*
  * Write the header file for 'mailbox'
  */
-int
-mailbox_write_header(mailbox)
-struct mailbox *mailbox;
+int mailbox_write_header(struct mailbox *mailbox)
 {
     int flag;
     FILE *newheader;
@@ -924,7 +1040,9 @@ struct mailbox *mailbox;
     }
 
     fputs(MAILBOX_HEADER_MAGIC, newheader);
-    fprintf(newheader, "%s\n", mailbox->quota.root ? mailbox->quota.root : "");
+    fprintf(newheader, "%s\t%s\n", 
+	    mailbox->quota.root ? mailbox->quota.root : "",
+	    mailbox->uniqueid);
     for (flag = 0; flag < MAX_USER_FLAGS; flag++) {
 	if (mailbox->flagname[flag]) {
 	    fprintf(newheader, "%s ", mailbox->flagname[flag]);
@@ -946,6 +1064,7 @@ struct mailbox *mailbox;
     }
 
     fclose(newheader);
+
     if (mailbox->header_fd != -1) {
 	close(mailbox->header_fd);
 	map_free(&mailbox->header_base, &mailbox->header_len);
@@ -967,8 +1086,7 @@ struct mailbox *mailbox;
  * Write the index header for 'mailbox'
  */
 int
-mailbox_write_index_header(mailbox)
-struct mailbox *mailbox;
+mailbox_write_index_header(struct mailbox *mailbox)
 {
     char buf[INDEX_HEADER_SIZE];
     int header_size = INDEX_HEADER_SIZE;
@@ -976,6 +1094,25 @@ struct mailbox *mailbox;
 
     assert(mailbox->index_lock_count != 0);
 
+    if (acappush_sock != -1) {
+	acapmbdata_t acapdata;
+
+	/* fill in structure */
+	strcpy(acapdata.name, mailbox->name);
+	acapdata.uidvalidity = mailbox->uidvalidity;
+	acapdata.exists      = mailbox->exists;
+	acapdata.deleted     = mailbox->deleted;
+	acapdata.answered    = mailbox->answered;
+	acapdata.flagged     = mailbox->flagged;
+	
+	/* send */
+	if (sendto(acappush_sock, &acapdata, 20+strlen(mailbox->name), 0,
+		   (struct sockaddr *) &acappush_remote, 
+		   acappush_remote_len) == -1) {
+	    syslog(LOG_ERR, "sending to acappush: %m");
+	}
+    }
+
     *((bit32 *)(buf+OFFSET_GENERATION_NO)) = mailbox->generation_no;
     *((bit32 *)(buf+OFFSET_FORMAT)) = htonl(mailbox->format);
     *((bit32 *)(buf+OFFSET_MINOR_VERSION)) = htonl(mailbox->minor_version);
@@ -987,6 +1124,9 @@ struct mailbox *mailbox;
     *((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED)) = htonl(mailbox->quota_mailbox_used);
     *((bit32 *)(buf+OFFSET_POP3_LAST_LOGIN)) = htonl(mailbox->pop3_last_login);
     *((bit32 *)(buf+OFFSET_UIDVALIDITY)) = htonl(mailbox->uidvalidity);
+    *((bit32 *)(buf+OFFSET_DELETED)) = htonl(mailbox->deleted);
+    *((bit32 *)(buf+OFFSET_ANSWERED)) = htonl(mailbox->answered);
+    *((bit32 *)(buf+OFFSET_FLAGGED)) = htonl(mailbox->flagged);
 
     if (mailbox->start_offset < header_size) header_size = mailbox->start_offset;
 
@@ -995,8 +1135,10 @@ struct mailbox *mailbox;
     if (n != header_size || fsync(mailbox->index_fd)) {
 	syslog(LOG_ERR, "IOERROR: writing index header for %s: %m",
 	       mailbox->name);
+	/* xxx can we unroll the acap send??? */
 	return IMAP_IOERROR;
     }
+
     return 0;
 }
 
@@ -1046,13 +1188,13 @@ struct index_record *record;
 
 /*
  * Append a new record to the index file
+ * call fsync() on index_fd if 'sync' is true
  */
-int
-mailbox_append_index(mailbox, record, start, num)
-struct mailbox *mailbox;
-struct index_record *record;
-unsigned start;
-unsigned num;
+int mailbox_append_index(struct mailbox *mailbox,
+			 struct index_record *record,
+			 unsigned start,
+			 unsigned num,
+			 int sync)
 {
     int i, j, len, n;
     char *buf, *p;
@@ -1089,7 +1231,7 @@ unsigned num;
     lseek(mailbox->index_fd, last_offset, SEEK_SET);
     n = retry_write(mailbox->index_fd, buf, len);
     free(buf);
-    if (n != len || fsync(mailbox->index_fd)) {
+    if (n != len || (sync && fsync(mailbox->index_fd))) {
 	syslog(LOG_ERR, "IOERROR: appending index records for %s: %m",
 	       mailbox->name);
 	ftruncate(mailbox->index_fd, last_offset);
@@ -1192,6 +1334,203 @@ struct quota *quota;
     return 0;
 }
 
+/*
+ * Lock the index file for 'mailbox'.  DON'T Reread index file header if necessary.
+ */
+int
+mailbox_lock_index_forcalc(mailbox)
+struct mailbox *mailbox;
+{
+    char fnamebuf[MAX_MAILBOX_PATH];
+    struct stat sbuffd, sbuffile;
+    int r;
+
+    if (mailbox->index_lock_count++) return 0;
+
+    assert(mailbox->quota.lock_count == 0);
+    assert(mailbox->seen_lock_count == 0);
+
+    strcpy(fnamebuf, mailbox->path);
+    strcat(fnamebuf, FNAME_INDEX);
+
+    for (;;) {
+	r = lock_blocking(mailbox->index_fd);
+	if (r == -1) {
+	    mailbox->index_lock_count--;
+	    syslog(LOG_ERR, "IOERROR: locking index for %s: %m",
+		   mailbox->name);
+	    return IMAP_IOERROR;
+	}
+
+	fstat(mailbox->index_fd, &sbuffd);
+	r = stat(fnamebuf, &sbuffile);
+	if (r == -1) {
+	    syslog(LOG_ERR, "IOERROR: stating index for %s: %m",
+		   mailbox->name);
+	    mailbox_unlock_index(mailbox);
+	    return IMAP_IOERROR;
+	}
+
+	if (sbuffd.st_ino == sbuffile.st_ino) break;
+
+	if ((r = mailbox_open_index(mailbox))) {
+	    return r;
+	}
+    }
+
+    return 0;
+}
+
+/*
+ * Calculate the number of messages in the mailbox with answered/deleted/flagged system
+ * flags
+ */
+
+static int mailbox_calculate_flagcounts(struct mailbox *mailbox)
+{
+    int r;
+    unsigned msgno;
+    bit32 numansweredflag = 0;
+    bit32 numdeletedflag = 0;
+    bit32 numflaggedflag = 0;
+    bit32 oldstart_offset, newstart_offset;
+    bit32 newversion;
+    struct stat sbuf;
+    char buf[INDEX_HEADER_SIZE];
+    unsigned long cache_len;
+    char fnamebuf[MAX_MAILBOX_PATH], fnamebufnew[MAX_MAILBOX_PATH];
+    FILE *newindex;
+    char *fnametail;
+    char *bufp;
+
+    /* Lock files and open new index/cache files */
+    r = mailbox_lock_header(mailbox);
+    if (r) return r;
+    r = mailbox_lock_index_forcalc(mailbox);
+    if (r) {
+	mailbox_unlock_header(mailbox);
+	return r;
+    }
+
+    r = mailbox_lock_pop(mailbox);
+    if (r) {
+	mailbox_unlock_index(mailbox);
+	mailbox_unlock_header(mailbox);
+	return r;
+    }
+
+    strcpy(fnamebuf, mailbox->path);
+    strcat(fnamebuf, FNAME_INDEX);
+    strcat(fnamebuf, ".NEW");
+    newindex = fopen(fnamebuf, "w+");
+    if (!newindex) {
+	syslog(LOG_ERR, "IOERROR: creating %s: %m", fnamebuf);
+	mailbox_unlock_pop(mailbox);
+	mailbox_unlock_index(mailbox);
+	mailbox_unlock_header(mailbox);
+	return IMAP_IOERROR;
+    }
+
+    /* Copy over old headers to new file */
+    memcpy(buf, mailbox->index_base, mailbox->start_offset);
+
+    /* insert new version number */
+    newversion = htonl(MAILBOX_MINOR_VERSION);
+    memcpy(buf+OFFSET_MINOR_VERSION, &newversion, sizeof(bit32));
+
+    /* save old start_offset; change start_offset */
+    oldstart_offset = mailbox->start_offset;
+    newstart_offset = htonl(INDEX_HEADER_SIZE);
+    memcpy(buf+OFFSET_START_OFFSET, &newstart_offset, sizeof(bit32));
+
+    fwrite(buf, 1, oldstart_offset, newindex);
+
+    if (fstat(mailbox->cache_fd, &sbuf) == -1) {
+	syslog(LOG_ERR, "IOERROR: fstating %s: %m", mailbox->name);
+	fatal("can't fstat cache file", EC_OSFILE);
+    }
+    cache_len = sbuf.st_size;
+    map_refresh(mailbox->cache_fd, 0, &mailbox->cache_base,
+		&mailbox->cache_len, cache_len, "cache", mailbox->name);
+
+    /* for each message look at the system flags */
+    for (msgno = 1; msgno <= mailbox->exists; msgno++) {
+	bit32 sysflags;
+	
+	bufp = (char *) (mailbox->index_base + mailbox->start_offset +
+			(msgno - 1)*mailbox->record_size);
+
+	/* Sanity check */
+	if (*((bit32 *)(bufp+OFFSET_UID)) == 0) {
+	    syslog(LOG_ERR, "IOERROR: %s zero index record %u/%u",
+		   mailbox->name, msgno, mailbox->exists);
+	    goto fail;
+	}
+
+	sysflags = ntohl(*((bit32 *)(bufp+OFFSET_SYSTEM_FLAGS)));
+	if (sysflags & FLAG_ANSWERED)
+	    numansweredflag++;
+	if (sysflags & FLAG_DELETED)
+	    numdeletedflag++;
+	if (sysflags & FLAG_FLAGGED)
+	    numflaggedflag++;
+    }
+
+    mailbox->answered = numansweredflag;
+    mailbox->deleted = numdeletedflag;
+    mailbox->flagged = numflaggedflag;
+
+    /* Grow the index header */
+    numansweredflag = htonl(numansweredflag);
+    fwrite(&numansweredflag, sizeof(bit32),1, newindex);
+    numdeletedflag = htonl(numdeletedflag);
+    fwrite(&numdeletedflag, sizeof(bit32),1, newindex);
+    numflaggedflag = htonl(numflaggedflag);
+    fwrite(&numflaggedflag, sizeof(bit32),1, newindex);
+
+    /* Write the rest of new index header same as old */
+    for (msgno = 1; msgno <= mailbox->exists; msgno++) {
+	bufp = (char *) (mailbox->index_base + oldstart_offset +
+			(msgno - 1)*mailbox->record_size);
+
+	fwrite(bufp, mailbox->record_size, 1, newindex);
+    }
+
+    /* Ensure everything made it to disk */
+    fflush(newindex);
+    if (ferror(newindex) ||
+	fsync(fileno(newindex))) {
+	syslog(LOG_ERR, "IOERROR: writing index for %s: %m",
+	       mailbox->name);
+	goto fail;
+    }
+
+    strcpy(fnamebuf, mailbox->path);
+    fnametail = fnamebuf + strlen(fnamebuf);
+    strcpy(fnametail, FNAME_INDEX);
+    strcpy(fnamebufnew, fnamebuf);
+    strcat(fnamebufnew, ".NEW");
+    if (rename(fnamebufnew, fnamebuf)) {
+	syslog(LOG_ERR, "IOERROR: renaming index file for %s: %m",
+	       mailbox->name);
+	goto fail;
+    }
+
+    mailbox_unlock_pop(mailbox);
+    mailbox_unlock_index(mailbox);
+    mailbox_unlock_header(mailbox);
+    fclose(newindex);
+
+    return 0;
+
+ fail:
+    mailbox_unlock_pop(mailbox);
+    mailbox_unlock_index(mailbox);
+    mailbox_unlock_header(mailbox);
+
+    return IMAP_IOERROR;
+}
+
 /*
  * Perform an expunge operation on 'mailbox'.  If 'iscurrentdir' is nonzero,
  * the current directory is set to the mailbox directory.  If nonzero, the
@@ -1211,7 +1550,13 @@ void *deciderock;
     FILE *newindex, *newcache;
     unsigned long *deleted;
     unsigned numdeleted = 0, quotadeleted = 0;
+    unsigned numansweredflag = 0;
+    unsigned numdeletedflag = 0;
+    unsigned numflaggedflag = 0;
     unsigned newexists;
+    unsigned newdeleted;
+    unsigned newanswered;
+    unsigned newflagged; 
     char *buf;
     unsigned msgno;
     int lastmsgdeleted = 1;
@@ -1239,7 +1584,7 @@ void *deciderock;
     }
 
     if (fstat(mailbox->cache_fd, &sbuf) == -1) {
-	syslog(LOG_ERR, "IOERROR: fstating %s: %m", fnamebuf);
+	syslog(LOG_ERR, "IOERROR: fstating %s: %m", fnamebuf); /* xxx is fnamebuf initialized??? */
 	fatal("can't fstat cache file", EC_OSFILE);
     }
     cache_len = sbuf.st_size;
@@ -1289,7 +1634,7 @@ void *deciderock;
     fwrite(buf, 1, mailbox->start_offset, newindex);
     /* Grow the index header if necessary */
     for (n = mailbox->start_offset; n < INDEX_HEADER_SIZE; n++) {
-	if (n == OFFSET_UIDVALIDITY+3) {
+	if (n == OFFSET_FLAGGED+3) {
 	    putc(1, newindex);
 	}
 	else {
@@ -1313,11 +1658,21 @@ void *deciderock;
 
 	if (decideproc ? decideproc(deciderock, buf) :
 	    (ntohl(*((bit32 *)(buf+OFFSET_SYSTEM_FLAGS))) & FLAG_DELETED)) {
+	    bit32 sysflags;
 
 	    /* Remember UID and size */
 	    deleted[numdeleted++] = ntohl(*((bit32 *)(buf+OFFSET_UID)));
 	    quotadeleted += ntohl(*((bit32 *)(buf+OFFSET_SIZE)));
 
+	    /* figure out if deleted msg has system flags. update counts accordingly */
+	    sysflags = ntohl(*((bit32 *)(buf+OFFSET_SYSTEM_FLAGS)));
+	    if (sysflags & FLAG_ANSWERED)
+		numansweredflag++;
+	    if (sysflags & FLAG_DELETED)
+		numdeletedflag++;
+	    if (sysflags & FLAG_FLAGGED)
+		numflaggedflag++;
+
 	    /* Copy over cache file data */
 	    if (!lastmsgdeleted) {
 		cache_offset = ntohl(*((bit32 *)(buf+OFFSET_CACHE_OFFSET)));
@@ -1362,6 +1717,14 @@ void *deciderock;
     /* Fix up exists */
     newexists = ntohl(*((bit32 *)(buf+OFFSET_EXISTS)))-numdeleted;
     *((bit32 *)(buf+OFFSET_EXISTS)) = htonl(newexists);
+    /* fix up other counts */
+    newanswered = ntohl(*((bit32 *)(buf+OFFSET_ANSWERED)))-numansweredflag;
+    *((bit32 *)(buf+OFFSET_ANSWERED)) = htonl(newanswered);
+    newdeleted = ntohl(*((bit32 *)(buf+OFFSET_DELETED)))-numdeletedflag;
+    *((bit32 *)(buf+OFFSET_DELETED)) = htonl(newdeleted);
+    newflagged = ntohl(*((bit32 *)(buf+OFFSET_FLAGGED)))-numflaggedflag;
+    *((bit32 *)(buf+OFFSET_FLAGGED)) = htonl(newflagged);
+
     /* Fix up quota_mailbox_used */
     *((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED)) =
       htonl(ntohl(*((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED)))-quotadeleted);
@@ -1422,8 +1785,25 @@ void *deciderock;
     }
 
     if (numdeleted) {
-	toimsp(mailbox->name, mailbox->uidvalidity,
-	       "UIDNnn", mailbox->last_uid, newexists, 0);
+	if (acappush_sock != -1) {
+	    acapmbdata_t acapdata;
+	    
+	    /* fill in structure */
+	    strcpy(acapdata.name, mailbox->name);
+	    acapdata.uidvalidity = mailbox->uidvalidity;
+	    acapdata.exists      = newexists;
+	    acapdata.deleted     = newdeleted;
+	    acapdata.answered    = newanswered;
+	    acapdata.flagged     = newflagged;		
+	    
+	    /* send */
+	    if (sendto(acappush_sock, &acapdata, 20+strlen(mailbox->name), 0,
+		       (struct sockaddr *) &acappush_remote, 
+		       acappush_remote_len) == -1) {
+		syslog(LOG_ERR, "Error sending to acappush: %m");
+		goto fail;
+	    }
+	}
     }
 
     mailbox_unlock_pop(mailbox);
@@ -1483,9 +1863,7 @@ char *indexbuf;
 /*
  * Expunge the expired news articles out of the netnews mailbox 'mailbox'
  */
-int
-mailbox_expungenews(mailbox)
-struct mailbox *mailbox;
+int mailbox_expungenews(struct mailbox *mailbox)
 {
     int r;
     struct newsexpunge newsexpunge;
@@ -1497,7 +1875,7 @@ struct mailbox *mailbox;
     unsigned long uid, miduid;
     char *p;
     int i;
-    int low, high, mid;
+    int low, high, mid = 0;
 
     assert(mailbox->format == MAILBOX_FORMAT_NETNEWS);
     assert(mailbox->index_lock_count == 0);
@@ -1522,7 +1900,7 @@ struct mailbox *mailbox;
 	}
 
 	group = mailbox->name;
-	if (newsprefixlen = strlen(config_getstring("newsprefix", ""))) {
+	if ((newsprefixlen = strlen(config_getstring("newsprefix", "")))) {
 	    group += newsprefixlen;
 	    if (*group == '.') group++;
 	}
@@ -1545,11 +1923,11 @@ struct mailbox *mailbox;
     }
     
     /* For each article in directory, mark it in msg_seen */
-    while (dirent = readdir(dirp)) {
-	if (!isdigit(dirent->d_name[0]) || dirent->d_name[0] == '0') continue;
+    while ((dirent = readdir(dirp))) {
+	if (!isdigit((int) (dirent->d_name[0])) || dirent->d_name[0] == '0') continue;
 	uid = 0;
 	p = dirent->d_name;
-	while (isdigit(*p)) {
+	while (isdigit((int) *p)) {
 	    uid = uid * 10 + *p++ - '0';
 	}
 	if (*p) continue;
@@ -1624,13 +2002,11 @@ const char *name;
 }
 
 
-int 
-mailbox_create(name, path, acl, format, mailboxp)
-const char *name;
-char *path;
-const char *acl;
-int format;
-struct mailbox *mailboxp;
+int mailbox_create(const char *name,
+		   char *path,
+		   const char *acl,
+		   int format,
+		   struct mailbox *mailboxp)
 {
     int r;
     char *p=path;
@@ -1641,7 +2017,7 @@ struct mailbox *mailboxp;
     int n;
     struct stat sbuf;
 
-    while (p = strchr(p+1, '/')) {
+    while ((p = strchr(p+1, '/'))) {
 	*p = '\0';
 	if (mkdir(path, 0777) == -1 && errno != EEXIST) {
 	    save_errno = errno;
@@ -1711,6 +2087,12 @@ struct mailbox *mailboxp;
     mailbox.quota_mailbox_used = 0;
     mailbox.pop3_last_login = 0;
     mailbox.uidvalidity = time(0);
+    mailbox.deleted = 0;
+    mailbox.answered = 0;
+    mailbox.flagged = 0;
+
+    mailbox.uniqueid = xmalloc(sizeof(char) * 32);
+    mailbox_make_uniqueid(mailbox.name, mailbox.uidvalidity, mailbox.uniqueid);
 
     r = mailbox_write_header(&mailbox);
     if (!r) r = mailbox_write_index_header(&mailbox);
@@ -1722,7 +2104,7 @@ struct mailbox *mailboxp;
 	    r = IMAP_IOERROR;
 	}
     }
-    if (!r) r = seen_create(&mailbox);
+    if (!r) r = seen_create_mailbox(&mailbox);
 
     if (mailboxp) {
 	*mailboxp = mailbox;
@@ -1737,9 +2119,7 @@ struct mailbox *mailboxp;
  * Delete and close the mailbox 'mailbox'.  Closes 'mailbox' whether
  * or not the deletion was successful.
  */
-int mailbox_delete(mailbox, delete_quota_root)
-struct mailbox *mailbox;
-int delete_quota_root;
+int mailbox_delete(struct mailbox *mailbox, int delete_quota_root)
 {
     int r;
     DIR *dirp;
@@ -1757,12 +2137,11 @@ int delete_quota_root;
 	return r;
     }
 
-    seen_delete(mailbox);
+    seen_delete_mailbox(mailbox);
 
     if (delete_quota_root) {
 	mailbox_delete_quota(&mailbox->quota);
-    }
-    else {
+    } else {
 	/* Free any quota being used by this mailbox */
 	if (mailbox->quota.used >= mailbox->quota_mailbox_used) {
 	    mailbox->quota.used -= mailbox->quota_mailbox_used;
@@ -1785,7 +2164,7 @@ int delete_quota_root;
     *tail++ = '/';
     dirp = opendir(mailbox->path);
     if (dirp) {
-	while (f = readdir(dirp)) {
+	while ((f = readdir(dirp))!=NULL) {
 	    if (f->d_name[0] == '.'
 		&& (f->d_name[1] == '\0'
 		    || (f->d_name[1] == '.' &&
@@ -1823,14 +2202,10 @@ char *indexbuf;
 }
 
 int
-mailbox_rename(oldname, newname, newpath, isinbox, olduidvalidityp,
-	       newuidvalidityp)
-const char *oldname;
-const char *newname;
-char *newpath;
-int isinbox;
-bit32 *olduidvalidityp;
-bit32 *newuidvalidityp;
+mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl, 
+	       const char *newname, char *newpath, int isinbox, 
+	       bit32 *olduidvalidityp, bit32 *newuidvalidityp,
+	       struct mailbox *mailboxp)
 {
     int r, r2;
     struct mailbox oldmailbox, newmailbox;
@@ -1840,14 +2215,13 @@ bit32 *newuidvalidityp;
     char *oldfnametail, *newfnametail;
 
     /* Open old mailbox and lock */
-    r = mailbox_open_header(oldname, 0, &oldmailbox);
-    if (r) {
-	return r;
-    }
+    mailbox_open_header_path(oldname, oldpath, oldacl, 0, &oldmailbox, 0);
+
     if (oldmailbox.format == MAILBOX_FORMAT_NETNEWS) {
 	mailbox_close(&oldmailbox);
 	return IMAP_MAILBOX_NOTSUPPORTED;
     }
+
     r =  mailbox_lock_header(&oldmailbox);
     if (!r) r = mailbox_open_index(&oldmailbox);
     if (!r) r = mailbox_lock_index(&oldmailbox);
@@ -1868,8 +2242,8 @@ bit32 *newuidvalidityp;
 	newmailbox.uidvalidity = oldmailbox.uidvalidity;
     }
 
-    *olduidvalidityp = oldmailbox.uidvalidity;
-    *newuidvalidityp = newmailbox.uidvalidity;
+    if (olduidvalidityp) *olduidvalidityp = oldmailbox.uidvalidity;
+    if (newuidvalidityp) *newuidvalidityp = newmailbox.uidvalidity;
 
     /* Copy flag names */
     for (flag = 0; flag < MAX_USER_FLAGS; flag++) {
@@ -1927,7 +2301,6 @@ bit32 *newuidvalidityp;
     newmailbox.index_fd = dup(oldmailbox.index_fd);
     (void) mailbox_read_index_header(&newmailbox);
     newmailbox.generation_no = oldmailbox.generation_no;
-    newmailbox.uidvalidity = *newuidvalidityp;
     (void) mailbox_write_index_header(&newmailbox);
 
     /* Copy over message files */
@@ -1955,8 +2328,7 @@ bit32 *newuidvalidityp;
 	/* Expunge old mailbox */
 	r = mailbox_expunge(&oldmailbox, 0, expungeall, (char *)0);
 	mailbox_close(&oldmailbox);
-    }
-    else {
+    } else {
 	r = mailbox_delete(&oldmailbox, 0);
     }
 
@@ -1975,7 +2347,11 @@ bit32 *newuidvalidityp;
     }
     if (r) goto fail;
 
-    mailbox_close(&newmailbox);
+    if (mailboxp) {
+	*mailboxp = newmailbox;
+    } else {
+	mailbox_close(&newmailbox);
+    }
     return 0;
 
  fail:
@@ -2057,7 +2433,7 @@ void mailbox_hash_mbox(char *buf,
 	    c = *(idx + 1);
 	}
 	c = (char) tolower((int) c);
-	if (!islower(c))
+	if (!islower((int) c))
 	    c = 'q';
 	
 	sprintf(buf, "%s/%c/%s", root, c, name);
@@ -2067,7 +2443,7 @@ void mailbox_hash_mbox(char *buf,
     }
 
     /* change all '.'s to '/' */
-    for (p = buf + strlen(root) + 3; *p; p++) {
+    for (p = buf + strlen(root) + 1; *p; p++) {
 	if (*p == '.') *p = '/';
     }
 }
@@ -2086,7 +2462,7 @@ void mailbox_hash_quota(char *buf, const char *qr)
 	c = *(idx + 1);
     }
     c = (char) tolower((int) c);
-    if (!islower(c))
+    if (!islower((int) c))
 	c = 'q';
 
     sprintf(buf, "%s%s%c/%s", config_dir, FNAME_QUOTADIR, c, qr);
