@@ -1,32 +1,49 @@
 /* mboxname.c -- Mailbox list manipulation routines
- $Id: mboxname.c,v 1.13 1999/04/08 21:04:26 tjs Exp $
- 
- # Copyright 1998 Carnegie Mellon University
- # 
- # No warranties, either expressed or implied, are made regarding the
- # operation, use, or results of the software.
- #
- # Permission to use, copy, modify and distribute this software and its
- # documentation is hereby granted for non-commercial purposes only
- # provided that this copyright notice appears in all copies and in
- # supporting documentation.
- #
- # Permission is also granted to Internet Service Providers and others
- # entities to use the software for internal purposes.
- #
- # The distribution, modification or sale of a product which uses or is
- # based on the software, in whole or in part, for commercial purposes or
- # benefits requires specific, additional permission from:
- #
- #  Office of Technology Transfer
- #  Carnegie Mellon University
- #  5000 Forbes Avenue
- #  Pittsburgh, PA  15213-3890
- #  (412) 268-4387, fax: (412) 268-7395
- #  tech-transfer@andrew.cmu.edu
+ $Id: mboxname.c,v 1.19 2000/05/23 20:52:24 robeson Exp $
+
+ * Copyright (c)1998-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
  *
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -34,12 +51,14 @@
 
 #include "assert.h"
 #include "glob.h"
-#include "config.h"
+#include "imapconf.h"
 #include "mailbox.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "xmalloc.h"
 
+#include "mboxname.h"
+
 /* Mailbox patterns which the design of the server prohibits */
 static char *badmboxpatterns[] = {
     "",
@@ -81,13 +100,9 @@ static const char index_mod64[256] = {
  * Convert the external mailbox 'name' to an internal name.
  * If 'userid' is non-null, it is the name of the current user.
  * On success, results are placed in the buffer pointed to by
- * 'result', the buffer must be of size MAX_MAILBOX_LEN+1.
+ * 'result', the buffer must be of size MAX_MAILBOX_NAME+1.
  */
-int
-mboxname_tointernal(name, userid, result)
-char *name;
-char *userid;
-char *result;
+int mboxname_tointernal(char *name, char *userid, char *result)
 {
     if ((name[0] == 'i' || name[0] == 'I') &&
 	!strncasecmp(name, "inbox", 5) &&
@@ -97,7 +112,7 @@ char *result;
 	    return IMAP_MAILBOX_BADNAME;
 	}
 
-	if (strlen(name)+strlen(userid) > MAX_MAILBOX_NAME) {
+	if (strlen(name)+strlen(userid)+5 > MAX_MAILBOX_NAME) {
 	    return IMAP_MAILBOX_BADNAME;
 	}
 
@@ -117,10 +132,7 @@ char *result;
 /*
  * Return nonzero if 'userid' owns the (internal) mailbox 'name'.
  */
-int
-mboxname_userownsmailbox(userid, name)
-char *userid;
-char *name;
+int mboxname_userownsmailbox(char *userid, char *name)
 {
     if (!strchr(userid, '.') && !strncmp(name, "user.", 5) &&
 	!strncmp(name+5, userid, strlen(userid)) &&
@@ -134,14 +146,12 @@ char *name;
  * Apply additional restrictions on netnews mailbox names.
  * Cannot have all-numeric name components.
  */
-int
-mboxname_netnewscheck(name)
-char *name;
+int mboxname_netnewscheck(char *name)
 {
     int c;
     int sawnonnumeric = 0;
 
-    while (c = *name++) {
+    while ((c = *name++)!=0) {
 	switch (c) {
 	case '.':
 	    if (!sawnonnumeric) return IMAP_MAILBOX_BADNAME;
@@ -175,9 +185,7 @@ char *name;
  * Restrictions are hardwired for now.
  */
 #define GOODCHARS " +,-.0123456789:=@ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz~"
-int
-mboxname_policycheck(name)
-char *name;
+int mboxname_policycheck(char *name)
 {
     int i;
     struct glob *g;
@@ -250,7 +258,7 @@ char *name;
 		    return IMAP_MAILBOX_BADNAME;
 		}
 		ucs4 = ((c6 & 0x0f) << 12) | (c7 << 6) | c8;
-		if ((ucs4 & 0xff00) == 0 || (ucs4 & 0xf800) == 0xd800) {
+		if ((ucs4 & 0xff80) == 0 || (ucs4 & 0xf800) == 0xd800) {
 		    /* US-ASCII or multi-word character */
 		    return IMAP_MAILBOX_BADNAME;
 		}
