@@ -1,39 +1,53 @@
 /* pop3d.c -- POP3 server protocol parsing
  *
- * Copyright 1998 Carnegie Mellon University
- * 
- * No warranties, either expressed or implied, are made regarding the
- * operation, use, or results of the software.
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
- * Permission to use, copy, modify and distribute this software and its
- * documentation is hereby granted for non-commercial purposes only
- * provided that this copyright notice appears in all copies and in
- * supporting documentation.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission is also granted to Internet Service Providers and others
- * entities to use the software for internal purposes.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * The distribution, modification or sale of a product which uses or is
- * based on the software, in whole or in part, for commercial purposes or
- * benefits requires specific, additional permission from:
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
- *  Office of Technology Transfer
- *  Carnegie Mellon University
- *  5000 Forbes Avenue
- *  Pittsburgh, PA  15213-3890
- *  (412) 268-4387, fax: (412) 268-7395
- *  tech-transfer@andrew.cmu.edu
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
- * $Id: pop3d.c,v 1.54 1999/12/29 19:20:55 leg Exp $
+ * $Id: pop3d.c,v 1.72 2000/05/29 03:02:59 leg Exp $
  */
+#include <config.h>
 
-#ifndef __GNUC__
-/* can't use attributes... */
-#define __attribute__(foo)
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
 #endif
-
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
@@ -46,25 +60,36 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <ctype.h>
+#include "prot.h"
 
 #include <sasl.h>
+#include <saslutil.h>
 
 #include "acl.h"
 #include "util.h"
 #include "auth.h"
-#include "config.h"
+#include "imapconf.h"
+#include "tls.h"
 
-/* openSSL has it's own DES function which conflict in names with those used by krb.h */
-#ifdef HAVE_SSL
-#undef HAVE_SSL
-#endif /* HAVE_SSL */
 
-#include "prot.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "version.h"
 #include "xmalloc.h"
+#include "mboxlist.h"
+
+#ifdef HAVE_KRB
+/* kerberos des is purported to conflict with OpenSSL DES */
+#define DES_DEFS
+#include <krb.h>
+
+/* MIT's kpop authentication kludge */
+char klrealm[REALM_SZ];
+AUTH_DAT kdata;
+#endif /* HAVE_KRB */
+static int kflag = 0;
 
 extern int optind;
 extern char *optarg;
@@ -72,16 +97,11 @@ extern int opterr;
 
 extern int errno;
 
-extern char *login_capabilities();
 
-#ifdef HAVE_KRB
-#include <krb.h>
 
-/* MIT's kpop authentication kludge */
-int kflag = 0;
-char klrealm[REALM_SZ];
-AUTH_DAT kdata;
-#endif
+#ifdef HAVE_SSL
+extern SSL *tls_conn;
+#endif /* HAVE_SSL */
 
 sasl_conn_t *popd_saslconn; /* the sasl connection context */
 
@@ -100,6 +120,8 @@ struct msg {
     int deleted;
 } *popd_msg;
 
+int popd_starttls_done = 0;
+
 static struct mailbox mboxstruct;
 
 static int expungedeleted();
@@ -108,122 +130,102 @@ static void cmd_auth();
 static void cmd_capa();
 static void cmd_pass();
 static void cmd_user();
+static void cmd_starttls(int pop3s);
+static int starttls_enabled(void);
+void eatline(void);
+static void blat(int msg,int lines);
+int openinbox(void);
+static void cmdloop(void);
+void kpop(void);
+static int parsenum(char **ptr);
+void usage(void);
 
-/* This creates a structure that defines the allowable
- *   security properties 
- */
-static sasl_security_properties_t *make_secprops(int min,int max)
-{
-  sasl_security_properties_t *ret=
-    (sasl_security_properties_t *) xmalloc(sizeof(sasl_security_properties_t));
-
-  ret->maxbufsize=4000;
-  ret->min_ssf=min;     /* minimum allowable security strength */
-  ret->max_ssf=max;     /* maximum allowable security strength */
-
-  ret->security_flags = 0;
-  if (!config_getswitch("allowplaintext", 1)) {
-      ret->security_flags |= SASL_SEC_NOPLAINTEXT;
-  }
-  if (!config_getswitch("allowanonymouslogin", 0)) {
-      ret->security_flags |= SASL_SEC_NOANONYMOUS;
-  }
-  ret->property_names=NULL;
-  ret->property_values=NULL;
-
-  return ret;
-}
-
-/* this is a wrapper to call the cyrus configuration from SASL */
-static int mysasl_config(void *context __attribute__((unused)), 
-			 const char *plugin_name,
-			 const char *option,
-			 const char **result,
-			 unsigned *len)
-{
-    char opt[1024];
-
-    if (strcmp(option, "srvtab")) { /* we don't transform srvtab! */
-	int sl = 5 + (plugin_name ? strlen(plugin_name) + 1 : 0);
-
-	strncpy(opt, "sasl_", 1024);
-	if (plugin_name) {
-	    strncat(opt, plugin_name, 1019);
-	    strncat(opt, "_", 1024 - sl);
-	}
- 	strncat(opt, option, 1024 - sl - 1);
-	opt[1023] = '\0';
-    } else {
-	strncpy(opt, option, 1024);
-    }
-
-    *result = config_getstring(opt, NULL);
-    if (*result != NULL) {
-	if (len) { *len = strlen(*result); }
-	return SASL_OK;
-    }
-   
-    return SASL_FAIL;
-}
+extern void setproctitle_init(int argc, char **argv, char **envp);
+extern int proc_register(char *progname, char *clienthost, 
+			 char *userid, char *mailbox);
+extern void proc_cleanup(void);
 
 static struct sasl_callback mysasl_cb[] = {
     { SASL_CB_GETOPT, &mysasl_config, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
-main(argc, argv, envp)
-int argc;
-char **argv;
-char **envp;
+/*
+ * run once when process is forked;
+ * MUST NOT exit directly; must return with non-zero error code
+ */
+int service_init(int argc, char **argv, char **envp)
 {
+    int r;
+
+    config_changeident("pop3d");
+    if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
+    setproctitle_init(argc, argv, envp);
+
+    signal(SIGPIPE, SIG_IGN);
+
+    /* set the SASL allocation functions */
+    sasl_set_alloc((sasl_malloc_t *) &xmalloc, 
+		   (sasl_calloc_t *) &calloc, 
+		   (sasl_realloc_t *) &xrealloc, 
+		   (sasl_free_t *) &free);
+
+    /* load the SASL plugins */
+    if ((r = sasl_server_init(mysasl_cb, "Cyrus")) != SASL_OK) {
+	syslog(LOG_ERR, "SASL failed initializing: sasl_server_init(): %s", 
+	       sasl_errstring(r, NULL, NULL));
+	return 2;
+    }
+
+    /* open the mboxlist, we'll need it for real work */
+    mboxlist_init(0);
+    mboxlist_open(NULL);
+
+    return 0;
+}
+
+/*
+ * run for each accepted connection
+ */
+int service_main(int argc, char **argv, char **envp)
+{
+    int pop3s = 0;
     int opt;
-    char hostname[MAXHOSTNAMELEN+1];
     int salen;
     struct hostent *hp;
-    struct sockaddr_in sa;
     int timeout;
     sasl_security_properties_t *secprops=NULL;
 
     popd_in = prot_new(0, 0);
     popd_out = prot_new(1, 1);
 
-    if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
-
-    opterr = 0;
-    while ((opt = getopt(argc, argv, "k")) != EOF) {
+    while ((opt = getopt(argc, argv, "sk")) != EOF) {
 	switch(opt) {
-#ifdef HAVE_KRB
+	case 's': /* pop3s (do starttls right away) */
+	    pop3s = 1;
+	    if (!starttls_enabled()) {
+		syslog(LOG_ERR, "pop3s: required OpenSSL options not present");
+		fatal("pop3s: required OpenSSL options not present",
+		      EC_CONFIG);
+	    }
 	case 'k':
 	    kflag++;
 	    break;
-#endif
-
 	default:
 	    usage();
 	}
     }
 
-    setproctitle_init(argc, argv, envp);
-    config_init("pop3d");
-
-    signal(SIGPIPE, SIG_IGN);
-    gethostname(hostname, sizeof(hostname));
-
     /* Find out name of client host */
     salen = sizeof(popd_remoteaddr);
     if (getpeername(0, (struct sockaddr *)&popd_remoteaddr, &salen) == 0 &&
 	popd_remoteaddr.sin_family == AF_INET) {
-	if (hp = gethostbyaddr((char *)&popd_remoteaddr.sin_addr,
-			       sizeof(popd_remoteaddr.sin_addr), AF_INET)) {
-	    if (strlen(hp->h_name) + 30 > sizeof(popd_clienthost)) {
-		strncpy(popd_clienthost, hp->h_name, sizeof(popd_clienthost)-30);
-		popd_clienthost[sizeof(popd_clienthost)-30] = '\0';
-	    }
-	    else {
-		strcpy(popd_clienthost, hp->h_name);
-	    }
-	}
-	else {
+	hp = gethostbyaddr((char *)&popd_remoteaddr.sin_addr,
+			   sizeof(popd_remoteaddr.sin_addr), AF_INET);
+	if (hp != NULL) {
+	    strncpy(popd_clienthost, hp->h_name, sizeof(popd_clienthost)-30);
+	    popd_clienthost[sizeof(popd_clienthost)-30] = '\0';
+	} else {
 	    popd_clienthost[0] = '\0';
 	}
 	strcat(popd_clienthost, "[");
@@ -235,24 +237,19 @@ char **envp;
 	}
     }
 
-    /* Make a SASL connection and setup some properties for it */
-
-    if (sasl_server_init(mysasl_cb, "Cyrus") != SASL_OK)
-	fatal("SASL failed initializing: sasl_server_init()",EC_TEMPFAIL); 
-
     /* other params should be filled in */
-    if (sasl_server_new("pop", hostname, NULL, 
-			NULL, 2000, &popd_saslconn)!=SASL_OK)
+    if (sasl_server_new("pop", config_servername, NULL, 
+			NULL, SASL_SECURITY_LAYER, &popd_saslconn) != SASL_OK)
 	fatal("SASL failed initializing: sasl_server_new()",EC_TEMPFAIL); 
 
     /* will always return something valid */
-    secprops=make_secprops(0,2000);        
+    secprops = mysasl_secprops(0);
     sasl_setprop(popd_saslconn, SASL_SEC_PROPS, secprops);
     
-    sasl_setprop(popd_saslconn,   SASL_IP_REMOTE, &popd_remoteaddr);  
-    sasl_setprop(popd_saslconn,   SASL_IP_LOCAL, &popd_localaddr);  
+    sasl_setprop(popd_saslconn, SASL_IP_REMOTE, &popd_remoteaddr);  
+    sasl_setprop(popd_saslconn, SASL_IP_LOCAL, &popd_localaddr);  
 
-    proc_register("pop3d", popd_clienthost, (char *)0, (char *)0);
+    proc_register("pop3d", popd_clienthost, NULL, NULL);
 
     /* Set inactivity timer */
     timeout = config_getint("poptimeout", 10);
@@ -260,25 +257,22 @@ char **envp;
     prot_settimeout(popd_in, timeout*60);
     prot_setflushonread(popd_in, popd_out);
 
-#ifdef HAVE_KRB
     if (kflag) kpop();
-#endif
+
+    /* we were connected on pop3s port so we should do 
+       TLS negotiation immediatly */
+    if (pop3s == 1) cmd_starttls(1);
 
     prot_printf(popd_out, "+OK %s Cyrus POP3 %s server ready\r\n",
-		hostname, CYRUS_VERSION);
-
+		config_servername, CYRUS_VERSION);
     cmdloop();
+
+    return 0;
 }
 
-usage()
+void usage(void)
 {
-    prot_printf(popd_out, "-ERR usage: pop3d%s\r\n",
-#ifdef HAVE_KRB
-		" [-k]"
-#else
-		""
-#endif
-		);
+    prot_printf(popd_out, "-ERR usage: pop3d [-k] [-s]\r\n");
     prot_flush(popd_out);
     exit(EC_USAGE);
 }
@@ -286,7 +280,8 @@ usage()
 /*
  * Cleanly shut down and exit
  */
-void shut_down(int code) __attribute__((noreturn));
+void shut_down(int code) __attribute__ ((noreturn));
+
 void shut_down(int code)
 {
     proc_cleanup();
@@ -317,7 +312,7 @@ void fatal(const char* s, int code)
  * MIT's kludge of a kpop protocol
  * Client does a krb_sendauth() first thing
  */
-kpop()
+void kpop(void)
 {
     Key_schedule schedule;
     KTEXT_ST ticket;
@@ -360,16 +355,21 @@ kpop()
 	shut_down(0);
     }
 }
+#else
+void kpop(void)
+{
+    usage();
+}
 #endif
 
 /*
  * Top-level command loop parsing
  */
-cmdloop()
+static void cmdloop(void)
 {
     char inputbuf[8192];
     char *p, *arg;
-    unsigned msg;
+    unsigned msg = 0;
 
     for (;;) {
 	if (!prot_fgets(inputbuf, sizeof(inputbuf), popd_in)) {
@@ -381,12 +381,12 @@ cmdloop()
 	if (p > inputbuf && p[-1] == '\r') *--p = '\0';
 
 	/* Parse into keword and argument */
-	for (p = inputbuf; *p && !isspace(*p); p++);
+	for (p = inputbuf; *p && !isspace((int) *p); p++);
 	if (*p) {
 	    *p++ = '\0';
 	    arg = p;
 	    if (strcasecmp(inputbuf, "pass") != 0) {
-		while (*arg && isspace(*arg)) {
+		while (*arg && isspace((int) *arg)) {
 		    arg++;
 		}
 	    }
@@ -429,6 +429,14 @@ cmdloop()
 		cmd_capa();
 	    }
 	}
+	else if (!strcmp(inputbuf, "stls") && starttls_enabled()) {
+	    if (arg) {
+		prot_printf(popd_out,
+			    "-ERR STLS doesn't take any arguements\r\n");
+	    } else {
+		cmd_starttls(0);
+	    }
+	}
 	else if (!popd_mailbox) {
 	    if (!strcmp(inputbuf, "user")) {
 		if (!arg) {
@@ -591,7 +599,8 @@ cmdloop()
 		prot_printf(popd_out, "+OK unique-id listing follows\r\n");
 		for (msg = 1; msg <= popd_exists; msg++) {
 		    if (!popd_msg[msg].deleted) {
-			prot_printf(popd_out, "%u %u\r\n", msg, popd_msg[msg].uid);
+			prot_printf(popd_out, "%u %u\r\n", msg, 
+				    popd_msg[msg].uid);
 		    }
 		}
 		prot_printf(popd_out, ".\r\n");
@@ -603,6 +612,110 @@ cmdloop()
     }		
 }
 
+#ifdef HAVE_SSL
+static int starttls_enabled(void)
+{
+    if (config_getstring("tls_cert_file", NULL) == NULL) return 0;
+    if (config_getstring("tls_key_file", NULL) == NULL) return 0;
+    return 1;
+}
+
+static void cmd_starttls(int pop3s)
+{
+    int result;
+    int *layerp;
+    sasl_external_properties_t external;
+
+
+    /* SASL and openssl have different ideas about whether ssf is signed */
+    layerp = (int *) &(external.ssf);
+
+    if (popd_starttls_done == 1)
+    {
+	prot_printf(popd_out, "-ERR %s\r\n", 
+		    "Already successfully executed STLS");
+	return;
+    }
+
+    result=tls_init_serverengine(5,        /* depth to verify */
+				 !pop3s,   /* can client auth? */
+				 0,        /* require client to auth? */
+				 (char *)config_getstring("tls_ca_file", ""),
+				 (char *)config_getstring("tls_ca_path", ""),
+				 (char *)config_getstring("tls_cert_file", ""),
+				 (char *)config_getstring("tls_key_file", ""));
+
+    if (result == -1) {
+
+	syslog(LOG_ERR, "[pop3d] error initializing TLS: "
+	       "[CA_file: %s] [CA_path: %s] [cert_file: %s] [key_file: %s]",
+	       (char *) config_getstring("tls_ca_file", ""),
+	       (char *) config_getstring("tls_ca_path", ""),
+	       (char *) config_getstring("tls_cert_file", ""),
+	       (char *) config_getstring("tls_key_file", ""));
+
+	if (pop3s == 0)
+	    prot_printf(popd_out, "-ERR %s\r\n", "Error initializing TLS");
+	else
+	    fatal("tls_init() failed",EC_TEMPFAIL);
+
+	return;
+    }
+
+    if (pop3s == 0)
+    {
+	prot_printf(popd_out, "+OK %s\r\n", "Begin TLS negotiation now");
+	/* must flush our buffers before starting tls */
+	prot_flush(popd_out);
+    }
+  
+    result=tls_start_servertls(0, /* read */
+			       1, /* write */
+			       layerp,
+			       &(external.auth_id));
+
+    /* if error */
+    if (result==-1) {
+	if (pop3s == 0) {
+	    prot_printf(popd_out, "-ERR Starttls failed\r\n");
+	    syslog(LOG_NOTICE, "[pop3d] STARTTLS failed: %s", popd_clienthost);
+	} else {
+	    syslog(LOG_NOTICE, "pop3s failed: %s", popd_clienthost);
+	    fatal("tls_start_servertls() failed", EC_TEMPFAIL);
+	}
+	return;
+    }
+
+    /* tell SASL about the negotiated layer */
+    result = sasl_setprop(popd_saslconn, SASL_SSF_EXTERNAL, &external);
+
+    if (result != SASL_OK) {
+	fatal("sasl_setprop() failed: cmd_starttls()", EC_TEMPFAIL);
+    }
+
+    /* if authenticated set that */
+    if (external.auth_id != NULL) {
+	popd_userid = external.auth_id;
+    }
+
+    /* tell the prot layer about our new layers */
+    prot_settls(popd_in, tls_conn);
+    prot_settls(popd_out, tls_conn);
+
+    popd_starttls_done = 1;
+}
+#else
+static int starttls_enabled(void)
+{
+    return 0;
+}
+
+static void cmd_starttls(int pop3s __attribute__((unused)))
+{
+    fatal("cmd_starttls() called, but no OpenSSL", EC_SOFTWARE);
+}
+#endif /* HAVE_SSL */
+
 void
 cmd_user(user)
 char *user;
@@ -622,8 +735,8 @@ char *user;
     if ((fd = open(shutdownfilename, O_RDONLY, 0)) != -1) {
 	shutdown_in = prot_new(fd, 0);
 	prot_fgets(buf, sizeof(buf), shutdown_in);
-	if (p = strchr(buf, '\r')) *p = 0;
-	if (p = strchr(buf, '\n')) *p = 0;
+	if ((p = strchr(buf, '\r'))!=NULL) *p = 0;
+	if ((p = strchr(buf, '\n'))!=NULL) *p = 0;
 
 	for(p = buf; *p == '['; p++); /* can't have [ be first char */
 	prot_printf(popd_out, "-ERR %s\r\n", p);
@@ -709,7 +822,7 @@ char *pass;
     else {
 	syslog(LOG_NOTICE, "login: %s %s plaintext %s",
 	       popd_clienthost, popd_userid, reply ? reply : "");
-	if (plaintextloginpause = config_getint("plaintextloginpause", 0)) {
+	if ((plaintextloginpause = config_getint("plaintextloginpause", 0))!=0) {
 	    sleep(plaintextloginpause);
 	}
     }
@@ -721,29 +834,26 @@ char *pass;
 void
 cmd_capa()
 {
-    int i;
     int minpoll = config_getint("popminpoll", 0) * 60;
     int expire = config_getint("popexpiretime", -1);
-    const char *capabilities, *next_capabilities;
-    char *sasllist;
-    
+    int mechcount;
+    char *mechlist;
+
     prot_printf(popd_out, "+OK List of capabilities follows\r\n");
 
-#if 0
-    /* SASL special case: print SASL, then a list of supported capabilities
-       (parsed from the IMAP-style login_capabilities function, then a CRLF */
+    /* SASL special case: print SASL, then a list of supported capabilities */
     if (sasl_listmech(popd_saslconn,
-			 "", /* should be id string */
-			 ""," ","",
-			 &sasllist,
-			 NULL,NULL)==SASL_OK)
-    {
-      prot_printf(popd_out, "SASL"); /* no \r\n */
-      prot_printf(popd_out," %s",sasllist);
-      prot_printf(popd_out, "\r\n");    
+		      NULL, /* should be id string */
+		      "SASL ", " ", "\r\n",
+		      &mechlist,
+		      NULL, &mechcount) == SASL_OK && mechcount > 0) {
+	prot_write(popd_out, mechlist, strlen(mechlist));
+	free(mechlist);
     }
-#endif
 
+    if (starttls_enabled()) {
+	prot_printf(popd_out, "STLS\r\n");
+    }
     if (expire < 0) {
 	prot_printf(popd_out, "EXPIRE NEVER\r\n");
     } else {
@@ -765,110 +875,111 @@ cmd_capa()
 }
 
 
-
-
-void
-cmd_auth(authtype)
-char *authtype;
+/* according to RFC 2449, since we advertise the "SASL" capability, we
+ * must accept an optional second argument of the initial client
+ * response (base64 encoded!).
+ */ 
+void cmd_auth(char *arg)
 {
     int sasl_result;
-    char * clientin;
+    static struct buf clientin;
     int clientinlen=0;
-    
+    char *authtype;
     char *serverout;
     unsigned int serveroutlen;
     const char *errstr;
-    
-    const char *errorstring = NULL;
 
-    char buf[MAX_MAILBOX_PATH];
-    char *p;
-    FILE *logfile;
+    /* if client didn't specify an argument we give them the list */
+    if (!arg) {
+	char *sasllist;
+	unsigned int mechnum;
 
-    int *ssfp;
-    char *ssfmsg=NULL;
-
-   
-    /* if client didn't specify an auth mechanism we give them the list */
-    if (!authtype) {
-      char *sasllist;
-      unsigned int mechnum;
-
-      prot_printf(popd_out, "+OK List of supported mechanisms follows\r\n");
+	prot_printf(popd_out, "+OK List of supported mechanisms follows\r\n");
       
-      /* SASL special case: print SASL, then a list of supported capabilities
-       * (parsed from the IMAP-style login_capabilities function, 
-       * then a CRLF */
-      if (sasl_listmech(popd_saslconn,
-			"", /* should be id string */
-			"","\r\n","\r\n",
-			&sasllist,
-			NULL,&mechnum) == SASL_OK) {
-	  if (mechnum>0) {
-	      prot_printf(popd_out,"%s",sasllist);
-	  }
-      }
+	/* CRLF seperated, dot terminated */
+	if (sasl_listmech(popd_saslconn, NULL,
+			  "", "\r\n", "\r\n",
+			  &sasllist,
+			  NULL, &mechnum) == SASL_OK) {
+	    if (mechnum>0) {
+		prot_printf(popd_out,"%s",sasllist);
+	    }
+	}
       
-      prot_printf(popd_out, ".\r\n");
-      
-      return;
+	prot_printf(popd_out, ".\r\n");
+      	return;
+    }
+
+    authtype = arg;
+    while (*arg && !isspace((int) *arg)) {
+	arg++;
+    }
+    if (isspace(*arg)) {
+	/* null terminate authtype, get argument */
+	*arg++ = '\0';
+    } else {
+	/* no optional client response */
+	arg = NULL;
+    }
+
+    /* if arg != NULL, it's an initial client response */
+    if (arg) {
+	int arglen = strlen(arg);
+
+	clientin.alloc = arglen + 1;
+	clientin.s = xmalloc(clientin.alloc);
+	sasl_result = sasl_decode64(arg, arglen, clientin.s, &clientinlen);
+    } else {
+	sasl_result = SASL_OK;
+	clientinlen = 0;
     }
 
     /* server did specify a command, so let's try to authenticate */
-
-    sasl_result = sasl_server_start(popd_saslconn, authtype,
-				    NULL, 0,
-				    &serverout, &serveroutlen,
-				    &errstr);    
-
+    if (sasl_result == SASL_OK || sasl_result == SASL_CONTINUE)
+	sasl_result = sasl_server_start(popd_saslconn, authtype,
+					clientin.s, clientinlen,
+					&serverout, &serveroutlen,
+					&errstr);
     /* sasl_server_start will return SASL_OK or SASL_CONTINUE on success */
-
     while (sasl_result == SASL_CONTINUE)
     {
+	/* print the message to the user */
+	printauthready(popd_out, serveroutlen, (unsigned char *)serverout);
+	free(serverout);
 
-      /* print the message to the user */
-      printauthready(serveroutlen, (unsigned char *)serverout);
-      free(serverout);
+	/* get string from user */
+	clientinlen = getbase64string(popd_in, &clientin);
+	if (clientinlen == -1) {
+	    prot_printf(popd_out, "-ERR Invalid base64 string\r\n");
+	    return;
+	}
 
-      /* get string from user */
-      clientinlen = readbase64string(&clientin);
-      if (clientinlen == -1) {
-	prot_printf(popd_out, "-ERR Invalid base64 string\r\n");
-	return;
-      }
-
-      sasl_result = sasl_server_step(popd_saslconn,
-				     clientin,
-				     clientinlen,
-				     &serverout, &serveroutlen,
-				     &errstr);
+	sasl_result = sasl_server_step(popd_saslconn,
+				       clientin.s,
+				       clientinlen,
+				       &serverout, &serveroutlen,
+				       &errstr);
     }
 
-
     /* failed authentication */
     if (sasl_result != SASL_OK)
     {
-	syslog(LOG_NOTICE, "badlogin: %s %s %i",
-	       popd_clienthost, authtype, sasl_result);
+	sleep(3);      
 	
-	if (clientin) {
+	/* convert the sasl error code to a string */
+	if (!errstr) errstr = sasl_errstring(sasl_result, NULL, NULL);
+	if (!errstr) errstr = "unknown error";
+	
+	prot_printf(popd_out, "-ERR authenticating: %s\r\n", errstr);
+
+	if (authtype) {
 	    syslog(LOG_NOTICE, "badlogin: %s %s %s",
-		   popd_clienthost, authtype, clientin);
+		   popd_clienthost, authtype, errstr);
 	} else {
 	    syslog(LOG_NOTICE, "badlogin: %s %s",
 		   popd_clienthost, authtype);
 	}
 	
-	sleep(3);      
-	
-	/* convert the sasl error code to a string */
-	errorstring = sasl_errstring(sasl_result, NULL, NULL);
-      
-	if (errorstring)
-	    prot_printf(popd_out, "-ERR %s\r\n",errorstring);
-	else
-	    prot_printf(popd_out, "-ERR Error Authenticating\r\n");
-	
 	return;
     }
 
@@ -877,41 +988,31 @@ char *authtype;
     /* get the userid from SASL --- already canonicalized from
      * mysasl_authproc()
      */
-
     sasl_result = sasl_getprop(popd_saslconn, SASL_USERNAME,
-			     (void **) &popd_userid);
-    if (sasl_result!=SASL_OK)
-    {
-      prot_printf(popd_out, "-ERR weird SASL error %d getting SASL_USERNAME\r\n", 
-		  sasl_result);
-      return;
+			       (void **) &popd_userid);
+    if (sasl_result != SASL_OK) {
+	prot_printf(popd_out, 
+		    "-ERR weird SASL error %d getting SASL_USERNAME\r\n", 
+		    sasl_result);
+	return;
     }
-
-    syslog(LOG_NOTICE, "login: %s %s %s sasl code=%d", popd_clienthost, popd_userid,
-	   authtype, sasl_result);
-
-    /* xxx here */
-    if (openinbox()==0)
-    {
-
-      proc_register("pop3d", popd_clienthost, popd_userid, popd_mailbox->name);    
-
-      syslog(LOG_NOTICE, "login: %s %s %s %s", popd_clienthost, popd_userid,
-	     authtype, "User logged in");
-
-
-      prot_setsasl(popd_in,  popd_saslconn);
-      prot_setsasl(popd_out, popd_saslconn);
-
-    }
-
-}
     
+    if (openinbox()==0) {
+	proc_register("pop3d", popd_clienthost, 
+		      popd_userid, popd_mailbox->name);    
+
+	syslog(LOG_NOTICE, "login: %s %s %s %s", popd_clienthost, popd_userid,
+	       authtype, "User logged in");
+
+	prot_setsasl(popd_in,  popd_saslconn);
+	prot_setsasl(popd_out, popd_saslconn);
+    }
+}
 
 /*
  * Complete the login process by opening and locking the user's inbox
  */
-int openinbox()
+int openinbox(void)
 {
     char inboxname[MAX_MAILBOX_PATH];
     int r, msg;
@@ -968,7 +1069,7 @@ int openinbox()
 	popd_highest = 0;
 	popd_msg = (struct msg *)xmalloc((popd_exists+1) * sizeof(struct msg));
 	for (msg = 1; msg <= popd_exists; msg++) {
-	    if (r = mailbox_read_index_record(&mboxstruct, msg, &record))
+	    if ((r = mailbox_read_index_record(&mboxstruct, msg, &record))!=0)
 	      break;
 	    popd_msg[msg].uid = record.uid;
 	    popd_msg[msg].size = record.size;
@@ -989,8 +1090,8 @@ int openinbox()
     proc_register("pop3d", popd_clienthost, popd_userid, popd_mailbox->name);
 
     /* Create telemetry log */
-    sprintf(buf, "%s%s%s/%u", config_dir, FNAME_LOGDIR, popd_userid,
-	    getpid());
+    sprintf(buf, "%s%s%s/%lu", config_dir, FNAME_LOGDIR, popd_userid,
+	    (long unsigned) getpid());
     logfile = fopen(buf, "w");
     if (logfile) {
 	prot_setlog(popd_in, fileno(logfile));
@@ -1001,9 +1102,7 @@ int openinbox()
     return 0;
 }
 
-blat(msg, lines)
-int msg;
-int lines;
+static void blat(int msg,int lines)
 {
     FILE *msgfile;
     char buf[4096];
@@ -1038,22 +1137,21 @@ int lines;
     prot_printf(popd_out, ".\r\n");
 }
 
-int parsenum(ptr)
-char **ptr;
+static int parsenum(char **ptr)
 {
     char *p = *ptr;
     int result = 0;
 
-    if (!isdigit(*p)) {
+    if (!isdigit((int) *p)) {
 	*ptr = 0;
 	return -1;
     }
-    while (*p && isdigit(*p)) {
+    while (*p && isdigit((int) *p)) {
 	result = result * 10 + *p++ - '0';
     }
 
     if (*p) {
-	while (*p && isspace(*p)) p++;
+	while (*p && isspace((int) *p)) p++;
 	*ptr = p;
     }
     else *ptr = 0;
@@ -1075,238 +1173,13 @@ char *index;
     return 0;
 }
 
-/*
- * Print an authentication ready response
- */
-static char basis_64[] =
-   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-printauthready(len, data)
-int len;
-unsigned char *data;
-{
-    int c1, c2, c3;
-
-    prot_putc('+', popd_out);
-    prot_putc(' ', popd_out);
-    while (len) {
-	c1 = *data++;
-	len--;
-	prot_putc(basis_64[c1>>2], popd_out);
-	if (len == 0) c2 = 0;
-	else c2 = *data++;
-	prot_putc(basis_64[((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4)], popd_out);
-	if (len == 0) {
-	    prot_putc('=', popd_out);
-	    prot_putc('=', popd_out);
-	    break;
-	}
-
-	if (--len == 0) c3 = 0;
-	else c3 = *data++;
-        prot_putc(basis_64[((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6)], popd_out);
-	if (len == 0) {
-	    prot_putc('=', popd_out);
-	    break;
-	}
-	
-	--len;
-        prot_putc(basis_64[c3 & 0x3F], popd_out);
-    }
-    prot_putc('\r', popd_out);
-    prot_putc('\n', popd_out);
-    prot_flush(popd_out);
-}
-
-#define XX 127
-/*
- * Table for decoding base64
- */
-static const char index_64[256] = {
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,62, XX,XX,XX,63,
-    52,53,54,55, 56,57,58,59, 60,61,XX,XX, XX,XX,XX,XX,
-    XX, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
-    15,16,17,18, 19,20,21,22, 23,24,25,XX, XX,XX,XX,XX,
-    XX,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
-    41,42,43,44, 45,46,47,48, 49,50,51,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-    XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
-};
-#define CHAR64(c)  (index_64[(unsigned char)(c)])
-
-#define BUFGROWSIZE 100
-
-/*
- * Parse a base64_string
- */
-int readbase64string(ptr)
-char **ptr;
-{
-    int c1, c2, c3, c4;
-    int i, len = 0;
-    static char *buf;
-    static int alloc = 0;
-
-    if (alloc == 0) {
-	alloc = BUFGROWSIZE;
-	buf = xmalloc(alloc+1);
-    }
-	
-    for (;;) {
-	c1 = prot_getc(popd_in);
-	if (c1 == '\r') {
-	    c1 = prot_getc(popd_in);
-	    if (c1 != '\n') {
-		eatline();
-		return -1;
-	    }
-	    *ptr = buf;
-	    return len;
-	}
-	else if (c1 == '\n') {
-	    *ptr = buf;
-	    return len;
-	}
-
-	if (CHAR64(c1) == XX) {
-	    eatline();
-	    return -1;
-	}
-	
-	c2 = prot_getc(popd_in);
-	if (CHAR64(c2) == XX) {
-	    if (c2 != '\n') eatline();
-	    return -1;
-	}
-
-	c3 = prot_getc(popd_in);
-	if (c3 != '=' && CHAR64(c3) == XX) {
-	    if (c3 != '\n') eatline();
-	    return -1;
-	}
-
-	c4 = prot_getc(popd_in);
-	if (c4 != '=' && CHAR64(c4) == XX) {
-	    if (c4 != '\n') eatline();
-	    return -1;
-	}
-
-	if (len+3 >= alloc) {
-	    alloc = len+BUFGROWSIZE;
-	    buf = xrealloc(buf, alloc+1);
-	}
-
-	buf[len++] = ((CHAR64(c1)<<2) | ((CHAR64(c2)&0x30)>>4));
-	if (c3 == '=') {
-	    c1 = prot_getc(popd_in);
-	    if (c1 == '\r') c1 = prot_getc(popd_in);
-	    if (c1 != '\n') {
-		eatline();
-		return -1;
-	    }
-	    if (c4 != '=') return -1;
-	    *ptr = buf;
-	    return len;
-	}
-	buf[len++] = (((CHAR64(c2)&0xf)<<4) | ((CHAR64(c3)&0x3c)>>2));
-	if (c4 == '=') {
-	    c1 = prot_getc(popd_in);
-	    if (c1 == '\r') c1 = prot_getc(popd_in);
-	    if (c1 != '\n') {
-		eatline();
-		return -1;
-	    }
-	    *ptr = buf;
-	    return len;
-	}
-	buf[len++] = (((CHAR64(c3)&0x3)<<6) | CHAR64(c4));
-    }
-}
-
-/*
- * Parse a base64_string
- */
-int parsebase64string(ptr, s)
-char **ptr;
-const char *s;
-{
-    int c1, c2, c3, c4;
-    int i, len = 0;
-    static char *buf;
-    static int alloc = 0;
-
-    if (alloc == 0) {
-	alloc = BUFGROWSIZE;
-	buf = xmalloc(alloc+1);
-    }
-	
-    for (;;) {
-	c1 = *s++;
-	if (c1 == '\0') {
-	    *ptr = buf;
-	    return len;
-	}
-
-	if (CHAR64(c1) == XX) {
-	    return -1;
-	}
-	
-	c2 = *s++;
-	if (CHAR64(c2) == XX) {
-	    return -1;
-	}
-
-	c3 = *s++;
-	if (c3 != '=' && CHAR64(c3) == XX) {
-	    return -1;
-	}
-
-	c4 = *s++;
-	if (c4 != '=' && CHAR64(c4) == XX) {
-	    return -1;
-	}
-
-	if (len+3 >= alloc) {
-	    alloc = len+BUFGROWSIZE;
-	    buf = xrealloc(buf, alloc+1);
-	}
-
-	buf[len++] = ((CHAR64(c1)<<2) | ((CHAR64(c2)&0x30)>>4));
-	if (c3 == '=') {
-	    c1 = *s++;
-	    if (c1 != '\0') {
-		return -1;
-	    }
-	    if (c4 != '=') return -1;
-	    *ptr = buf;
-	    return len;
-	}
-	buf[len++] = (((CHAR64(c2)&0xf)<<4) | ((CHAR64(c3)&0x3c)>>2));
-	if (c4 == '=') {
-	    c1 = *s++;
-	    if (c1 != '\0') {
-		return -1;
-	    }
-	    *ptr = buf;
-	    return len;
-	}
-	buf[len++] = (((CHAR64(c3)&0x3)<<6) | CHAR64(c4));
-    }
-}
-
 /*
  * Eat characters up to and including the next newline
  */
-eatline()
+void eatline(void)
 {
     int c;
 
-    while ((c = prot_getc(popd_in)) != EOF && c != '\n');
+    while ((c = prot_getc(popd_in)) != EOF && c != '\n')
+    { }
 }
