@@ -1,32 +1,53 @@
 /* quota.c -- program to report/reconstruct quotas
  *
- * Copyright 1998 Carnegie Mellon University
- * 
- * No warranties, either expressed or implied, are made regarding the
- * operation, use, or results of the software.
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
- * Permission to use, copy, modify and distribute this software and its
- * documentation is hereby granted for non-commercial purposes only
- * provided that this copyright notice appears in all copies and in
- * supporting documentation.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission is also granted to Internet Service Providers and others
- * entities to use the software for internal purposes.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * The distribution, modification or sale of a product which uses or is
- * based on the software, in whole or in part, for commercial purposes or
- * benefits requires specific, additional permission from:
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
- *  Office of Technology Transfer
- *  Carnegie Mellon University
- *  5000 Forbes Avenue
- *  Pittsburgh, PA  15213-3890
- *  (412) 268-4387, fax: (412) 268-7395
- *  tech-transfer@andrew.cmu.edu
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $Id: quota.c,v 1.29 1999/08/20 20:08:30 leg Exp $ */
+/* $Id: quota.c,v 1.34 2000/05/23 20:52:30 robeson Exp $ */
 
+
+#include <config.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -54,16 +75,31 @@
 #endif
 
 #include "assert.h"
-#include "config.h"
+#include "imapconf.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
+#include "mboxlist.h"
+#include "convert_code.h"
 
 extern int errno;
 extern int optind;
 extern char *optarg;
 
+/* forward declarations */
+void usage(void);
+void reportquota(void);
+int buildquotalist(char **roots, int nroots);
+int fixquota_mailbox(char *name,
+		     int matchlen,
+		     int maycreate,
+		     void* rock);
+int fixquota(int ispartial);
+int fixquota_fixroot(struct mailbox *mailbox,
+		     char *root);
+int fixquota_finish(int thisquota);
+
 extern char *mailbox_findquota();
 
 struct quotaentry {
@@ -83,9 +119,7 @@ int firstquota;
 int redofix;
 int partial;
 
-main(argc, argv)
-int argc;
-char **argv;
+int main(int argc,char **argv)
 {
     int opt;
     int fflag = 0;
@@ -118,9 +152,11 @@ char **argv;
     }
 
     exit(code);
+
+    return 0;
 }
 
-usage()
+void usage(void)
 {
     fprintf(stderr, "usage: quota [-f] [prefix]...\n");
     exit(EC_USAGE);
@@ -139,10 +175,7 @@ char *a, *b;
 /*
  * Build the list of quota roots in 'quota'
  */
-int 
-buildquotalist(roots, nroots)
-char **roots;
-int nroots;
+int buildquotalist(char **roots, int nroots)
 {
     int r;
     char quota_path[MAX_MAILBOX_PATH];
@@ -160,13 +193,13 @@ int nroots;
     if (!topp) {
 	return IMAP_IOERROR;
     }
-    while (dirent = readdir(topp)) {
+    while ((dirent = readdir(topp))!=NULL) {
 	if (dirent->d_name[0] == '.') continue;
 	
 	dirp = opendir(dirent->d_name);
 	if (!dirp) continue;
 
-	while (dirent = readdir(dirp)) {
+	while ((dirent = readdir(dirp))!=NULL) {
 	    if (dirent->d_name[0] == '.') continue;
 
 	    /* If restricting our list, see if this quota file matches */
@@ -219,12 +252,10 @@ int nroots;
 /*
  * Account for mailbox 'name' when fixing the quota roots
  */
-int
-fixquota_mailbox(name, matchlen, maycreate, rock)
-char *name;
-int matchlen;
-int maycreate;
-void* rock;
+int fixquota_mailbox(char *name,
+		     int matchlen,
+		     int maycreate,
+		     void* rock)
 {
     int r;
     struct mailbox mailbox;
@@ -298,10 +329,8 @@ void* rock;
     return 0;
 }
 	
-int
-fixquota_fixroot(mailbox, root)
-struct mailbox *mailbox;
-char *root;
+int fixquota_fixroot(struct mailbox *mailbox,
+		     char *root)
 {
     int i, r;
 
@@ -346,9 +375,7 @@ char *root;
 /*
  * Finish fixing up a quota root
  */
-int
-fixquota_finish(thisquota)
-int thisquota;
+int fixquota_finish(int thisquota)
 {
     int r;
 
@@ -372,7 +399,7 @@ int thisquota;
     }
     
     if (quota[thisquota].quota.used != quota[thisquota].newused) {
-	printf("%s: usage was %u, now %u\n", quota[thisquota].quota.root,
+	printf("%s: usage was %lu, now %lu\n", quota[thisquota].quota.root,
 	       quota[thisquota].quota.used, quota[thisquota].newused);
 	quota[thisquota].quota.used = quota[thisquota].newused;
 	r = mailbox_write_quota(&quota[thisquota].quota);
@@ -388,19 +415,16 @@ int thisquota;
 /*
  * Fix all the quota roots
  */
-int
-fixquota(ispartial)
-int ispartial;
+int fixquota(int ispartial)
 {
-    int r;
+    int r = 0;
     static char pattern[2] = "*";
 
     /*
      * Lock mailbox list to prevent mailbox creation/deletion
      * during the fix
      */
-    r = mboxlist_openlock();
-    if (r) return r;
+    mboxlist_open(NULL);
 
     redofix = 1;
     while (redofix) {
@@ -410,28 +434,28 @@ int ispartial;
 
 	r = mboxlist_findall(pattern, 1, 0, 0, fixquota_mailbox, NULL);
 	if (r) {
-	    mboxlist_unlock();
+	    mboxlist_close();
 	    return r;
 	}
 
 	while (firstquota < quota_num) {
 	    r = fixquota_finish(firstquota++);
 	    if (r) {
-		mboxlist_unlock();
+		mboxlist_close();
 		return r;
 	    }
 	}
     }
     
-    mboxlist_unlock();
+    mboxlist_close();
     return 0;
 }
     
 /*
  * Print out the quota report
  */
-int
-reportquota()
+void
+reportquota(void)
 {
     int i;
 
@@ -440,7 +464,7 @@ reportquota()
     for (i = 0; i < quota_num; i++) {
 	if (quota[i].deleted) continue;
 	if (quota[i].quota.limit > 0) {
-	    printf(" %7d %7d", quota[i].quota.limit,
+	    printf(" %7d %7ld", quota[i].quota.limit,
 		   ((quota[i].quota.used / QUOTA_UNITS) * 100) / quota[i].quota.limit);
 	}
 	else if (quota[i].quota.limit == 0) {
@@ -449,7 +473,7 @@ reportquota()
 	else {
 	    printf("                ");
 	}
-	printf(" %7d %s\n", quota[i].quota.used / QUOTA_UNITS,
+	printf(" %7ld %s\n", quota[i].quota.used / QUOTA_UNITS,
 	       quota[i].quota.root);
     }
 }
