@@ -1,39 +1,54 @@
 /* imtest.c -- imap test client
  * Tim Martin (SASL implementation)
- * $Id: imtest.c,v 1.42 2000/01/04 20:52:02 leg Exp $
+ * $Id: imtest.c,v 1.53 2000/05/29 02:56:19 leg Exp $
  *
- * Copyright 1999 Carnegie Mellon University
- * 
- * No warranties, either expressed or implied, are made regarding the
- * operation, use, or results of the software.
+ * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
- * Permission to use, copy, modify and distribute this software and its
- * documentation is hereby granted for non-commercial purposes only
- * provided that this copyright notice appears in all copies and in
- * supporting documentation.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission is also granted to Internet Service Providers and others
- * entities to use the software for internal purposes.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * The distribution, modification or sale of a product which uses or is
- * based on the software, in whole or in part, for commercial purposes or
- * benefits requires specific, additional permission from:
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
- *  Office of Technology Transfer
- *  Carnegie Mellon University
- *  5000 Forbes Avenue
- *  Pittsburgh, PA  15213-3890
- *  (412) 268-4387, fax: (412) 268-7395
- *  tech-transfer@andrew.cmu.edu
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <config.h>
+
 #include <sys/types.h>
 #include <sys/ipc.h>
 #include <sys/msg.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -55,6 +70,7 @@
 #include "prot.h"
 
 #ifdef HAVE_SSL
+#include <openssl/ssl.h>
 
 static SSL_CTX *tls_ctx = NULL;
 static SSL *tls_conn = NULL;
@@ -675,7 +691,7 @@ void interaction (int id, const char *prompt,
 	int c;
 	
 	printf("%s: ",prompt);
-	fgets(result, 1023, stdin);
+	fgets(result, sizeof(result) - 1, stdin);
 	c = strlen(result);
 	result[c - 1] = '\0';
     }
@@ -700,44 +716,51 @@ void fillin_interactions(sasl_interact_t *tlist)
 
 static int waitfor(char *tag)
 {
-  char *str=malloc(301);
-  char *ptr;
+    char str[1024];
+    char *ptr;
 
-  do {
-      str=prot_fgets(str,300,pin);
-      if (str == NULL) {
-	  imtest_fatal("prot layer failure");
-      }
-      printf("%s", str);
-  } while (strncmp(str, tag, strlen(tag)));
+    do {
+	if (prot_fgets(str,sizeof(str),pin) == NULL) {
+	    imtest_fatal("prot layer failure");
+	}
+	printf("%s", str);
+    } while (strncmp(str, tag, strlen(tag)));
 
-  free(str);
-
-  return 0;
+    return 0;
 }
 
 static int auth_login(void)
 {
+  char str[1024];
   /* we need username and password to do "login" */
   char *username;
   unsigned int userlen;
   char *pass;
   unsigned int passlen;
+  char *tag = "L01 ";
 
-  interaction(SASL_CB_AUTHNAME,"Userid",&username,&userlen);
+  interaction(SASL_CB_AUTHNAME,"Authname",&username,&userlen);
   interaction(SASL_CB_PASS,"Password",&pass,&passlen);
 
-  prot_printf(pout,"L01 LOGIN %s {%d}\r\n",username,passlen);
+  prot_printf(pout,"%sLOGIN %s {%d}\r\n", tag, username, passlen);
   prot_flush(pout);
 
   waitfor("+");
-
   prot_printf(pout,"%s\r\n",pass);
   prot_flush(pout);
-  waitfor("L01");
 
+  do {
+      if (prot_fgets(str,sizeof(str),pin) == NULL) {
+	  imtest_fatal("prot layer failure");
+      }
+      printf("%s", str);
+  } while (strncmp(str, tag, strlen(tag)));
 
-  return IMTEST_OK;
+  if (!strncasecmp(str + 4, "OK", 2)) {
+      return IMTEST_OK;
+  } else {
+      return IMTEST_FAIL;
+  }
 }
 
 int auth_sasl(char *mechlist)
@@ -749,7 +772,7 @@ int auth_sasl(char *mechlist)
   char *in;
   int inlen;
   const char *mechusing;
-  char inbase64[2048];
+  char inbase64[4096];
   int inbase64len;
 
   imt_stat status = STAT_CONT;
@@ -894,7 +917,7 @@ static char *parsemechlist(char *str)
 
 static char *ask_capability(int *supports_starttls)
 {
-  char *str=malloc(301);
+  char str[1024];
   char *ret;
 
   /* request capabilities of server */
@@ -904,9 +927,9 @@ static char *ask_capability(int *supports_starttls)
   printf("C: %s", CAPABILITY);
 
   do { /* look for the * CAPABILITY response */
-      str = prot_fgets(str,300,pin);
-      if (str == NULL) imtest_fatal("prot layer failure");
-  
+      if (prot_fgets(str,sizeof(str),pin) == NULL) {
+	  imtest_fatal("prot layer failure");
+      }
       printf("S: %s", str);
   } while (strncasecmp(str, "* CAPABILITY", 12));
 
@@ -919,14 +942,13 @@ static char *ask_capability(int *supports_starttls)
   ret=parsemechlist(str);
 
   do { /* look for TAG */
-      str = prot_fgets(str,300,pin);
-      if (str == NULL) imtest_fatal("prot layer failure");
+      if (prot_fgets(str, sizeof(str), pin) == NULL) {
+	  imtest_fatal("prot layer failure");
+      }
  
       printf("S: %s",str);
   } while (strncmp(str, CAPATAG, strlen(CAPATAG)));
 
-  free(str);
-
   return ret;
 }
 
@@ -1003,7 +1025,7 @@ static void send_recv_test(void)
 
 void interactive(char *filename)
 {
-  char buf[64];
+  char buf[2048];
   fd_set read_set, rset;
   fd_set write_set, wset;
   int nfds;
@@ -1046,7 +1068,6 @@ void interactive(char *filename)
       }
 
       if ((FD_ISSET(0, &rset)) && (FD_ISSET(sock, &wset)))  {
-	  
 	  if (fgets(buf, sizeof (buf) - 1, stdin) == NULL) {
 	      printf(LOGOUT);
 	      prot_write(pout, LOGOUT, sizeof (LOGOUT));
@@ -1082,6 +1103,7 @@ void interactive(char *filename)
 	  /* read from disk */	
 	  int numr = read(fd, buf, sizeof(buf));
 
+
 	  /* and send out over wire */
 	  if (numr < 0)
 	  {
@@ -1097,6 +1119,8 @@ void interactive(char *filename)
 	      prot_write(pout, LOGOUT, sizeof (LOGOUT));	      
 	      prot_flush(pout);
 	  } else {
+	      /* echo for the user */
+	      write(1, buf, numr);
 	      prot_write(pout, buf, numr);
 	      prot_flush(pout);
 	  }
@@ -1279,7 +1303,11 @@ int main(int argc, char **argv)
 	  result = auth_sasl(mechanism);
       }
   } else {
-      result = auth_sasl(mechlist);
+      if (*mechlist) {
+	  result = auth_sasl(mechlist);
+      } else {
+	  result = auth_login();
+      }
   }
 
   if (result == IMTEST_OK) {
