@@ -1,30 +1,50 @@
-/* prot.c -- stdio-like module that handles IMAP protection mechanisms
+/* prot.c -- stdio-like module that handles SASL protection mechanisms
  *
- *        Copyright 1998 by Carnegie Mellon University
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
- *                      All Rights Reserved
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appear in all copies and that
- * both that copyright notice and this permission notice appear in
- * supporting documentation, and that the name of CMU not be
- * used in advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
- * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
- * CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
- * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
  *
  */
 /*
- * $Id: prot.c,v 1.45 1999/12/23 18:58:53 leg Exp $
+ * $Id: prot.c,v 1.52 2000/05/23 20:56:19 robeson Exp $
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
@@ -46,6 +66,7 @@
 
 #include "prot.h"
 #include "xmalloc.h"
+#include "assert.h"
 
 /*
  * Create a new protection stream for file descriptor 'fd'.  Stream
@@ -58,7 +79,8 @@ int write;
     struct protstream *newstream;
 
     newstream = (struct protstream *) xmalloc(sizeof(struct protstream));
-    newstream->buf = (char *) xmalloc(sizeof(char) * (PROT_BUFSIZE + 4));
+    newstream->buf = (unsigned char *) 
+	xmalloc(sizeof(char) * (PROT_BUFSIZE + 4));
     newstream->buf_size = PROT_BUFSIZE;
     newstream->ptr = newstream->buf;
     newstream->cnt = write ? PROT_BUFSIZE : 0;
@@ -67,8 +89,6 @@ int write;
     newstream->write = write;
     newstream->logfd = -1;
     newstream->log_timeptr = 0;
-    newstream->func = 0;
-    newstream->state = 0;
     newstream->error = 0;
     newstream->eof = 0;
     newstream->read_timeout = 0;
@@ -89,8 +109,7 @@ int write;
 /*
  * Free a protection stream
  */
-int prot_free(s)
-struct protstream *s;
+int prot_free(struct protstream *s)
 {
     free(s->buf);
     free((char*)s);
@@ -111,9 +130,7 @@ int fd;
 /*
  * Start logging timing information for stream 's'.
  */
-int prot_setlogtime(s, ptr)
-struct protstream *s;
-time_t *ptr;
+int prot_setlogtime(struct protstream *s, time_t *ptr)
 {
     s->log_timeptr = ptr;
     time(s->log_timeptr);
@@ -188,10 +205,10 @@ sasl_conn_t *conn;
  * Set the read timeout for the stream 's' to 'timeout' seconds.
  * 's' must have been created for reading.
  */
-int prot_settimeout(s, timeout)
-struct protstream *s;
-int timeout;
+int prot_settimeout(struct protstream *s, int timeout)
 {
+    assert(!s->write);
+
     s->read_timeout = timeout;
     return 0;
 }
@@ -201,10 +218,10 @@ int timeout;
  * blocking for reading. 's' must have been created for reading,
  * 'flushs' for writing.
  */
-int prot_setflushonread(s, flushs)
-struct protstream *s;
-struct protstream *flushs;
+int prot_setflushonread(struct protstream *s, struct protstream *flushs)
 {
+    assert(!s->write);
+
     s->flushonread = flushs;
     return 0;
 }
@@ -213,11 +230,11 @@ struct protstream *flushs;
  * Set on stream 's' the callback 'proc' and 'rock'
  * to make the next time we have to wait for input.
  */
-int prot_setreadcallback(s, proc, rock)
-struct protstream *s;
-prot_readcallback_t *proc;
-void *rock;
+int prot_setreadcallback(struct protstream *s, 
+			 prot_readcallback_t *proc, void *rock)
 {
+    assert(!s->write);
+
     s->readcallback_proc = proc;
     s->readcallback_rock = rock;
     return 0;
@@ -228,8 +245,7 @@ void *rock;
  * error encountered on 's'.  If there is no error condition, return a
  * null pointer.
  */
-const char *prot_error(s)
-struct protstream *s;
+const char *prot_error(struct protstream *s)
 {
     return s->error;
 }
@@ -237,10 +253,10 @@ struct protstream *s;
 /*
  * Rewind the stream 's'.  's' must have been created for reading.
  */
-int 
-prot_rewind(s)
-struct protstream *s;
+int prot_rewind(struct protstream *s)
 {
+    assert(!s->write);
+
     if (lseek(s->fd, 0L, 0) == -1) {
 	s->error = strerror(errno);
 	return EOF;
@@ -255,18 +271,18 @@ struct protstream *s;
  * Read data into the empty buffer for the stream 's' and return the
  * first character.  Returns EOF on EOF or error.
  */
-int 
-prot_fill(s)
-struct protstream *s;
+int prot_fill(struct protstream *s)
 {
     int n;
-    char *ptr;
+    unsigned char *ptr;
     int left;
     int r;
     struct timeval timeout;
     fd_set rfds;
     int haveinput; 
    
+    assert(!s->write);
+
     if (s->eof || s->error) return EOF;
 
     do {
@@ -314,24 +330,17 @@ struct protstream *s;
 	
 	do {
 #ifdef HAVE_SSL	  
-
-	  /* just do a SSL read instead if we're under a tls layer */
-	  if (s->tls_conn!=NULL)
-	  {
-	    n = SSL_read(s->tls_conn, s->buf, PROT_BUFSIZE);
-	  } else {
-	    n = read(s->fd, s->buf, PROT_BUFSIZE);
-	  }
-
+	    /* just do a SSL read instead if we're under a tls layer */
+	    if (s->tls_conn != NULL) {
+		n = SSL_read(s->tls_conn, (char *) s->buf, PROT_BUFSIZE);
+	    } else {
+		n = read(s->fd, s->buf, PROT_BUFSIZE);
+	    }
 #else  /* HAVE_SSL */
-	  n = read(s->fd, s->buf, PROT_BUFSIZE);
+	    n = read(s->fd, s->buf, PROT_BUFSIZE);
 #endif /* HAVE_SSL */
-
-
-	    
 	} while (n == -1 && errno == EINTR);
-	
-	
+		
 	if (n <= 0) {
 	    if (n) s->error = strerror(errno);
 	    else s->eof = 1;
@@ -342,22 +351,23 @@ struct protstream *s;
 	    int result;
 	    char *out;
 	    unsigned outlen;
+	    static char errbuf[256];
 	    
 	    /* Decode the input token */
 	    result = sasl_decode(s->conn, (const char *) s->buf, n, 
 				 &out, &outlen);
 	    
 	    if (result != SASL_OK) {
-		snprintf(s->buf, 200, "Decoding error: %s (%i)",
+		snprintf(errbuf, 256, "Decoding error: %s (%i)",
 			 sasl_errstring(result, NULL, NULL), result);
-		s->error = s->buf;
+		s->error = errbuf;
 		return EOF;
 	    }
 	    
 	    if (outlen > 0) {
 		if (outlen > s->buf_size) {
-		    s->buf = (char *) xrealloc(s->buf, 
-					       sizeof(char) * (outlen + 4));
+		    s->buf = (unsigned char *) 
+			xrealloc(s->buf, sizeof(char) * (outlen + 4));
 		    s->buf_size = outlen;
 		}
 		memcpy(s->buf, out, outlen);
@@ -374,8 +384,6 @@ struct protstream *s;
 	    s->cnt = n;
 	}
 	
-	  
-
 	if (s->cnt > 0) {
 	    if (s->logfd != -1) {
 		time_t newtime;
@@ -388,7 +396,7 @@ struct protstream *s;
 		    *s->log_timeptr = newtime;
 		}
 
-		left = s->cnt+1;
+		left = s->cnt;
 		ptr = s->buf;
 		do {
 		    n = write(s->logfd, ptr, left);
@@ -404,21 +412,21 @@ struct protstream *s;
 	    s->cnt--;		/* we return the first char */
 	    return *s->buf;
 	}
-
     } while (1);
-
 }
 
 /*
  * Write out any buffered data in the stream 's'
  */
-int prot_flush(s)
-struct protstream *s;
+int prot_flush(struct protstream *s)
 {
-    char *ptr = s->buf;
+    unsigned char *ptr = s->buf;
     int left = s->ptr - s->buf;
     int n;
-    char *foo;
+    char *encoded_output;
+
+    assert(s->write);
+    assert(s->cnt >= 0);
 
     if (s->eof || s->error) {
 	s->ptr = s->buf;
@@ -451,34 +459,31 @@ struct protstream *s;
 	ptr = s->buf;
     }
 
-
-
-    if (s->saslssf!=0) {
-      /* Encode the data */  /* xxx handle left */
-      unsigned int outlen;
-      int result;
-
-      result=sasl_encode(s->conn, ptr, left, &foo, &outlen);
-      if (result!=SASL_OK)
-      {
-	s->error = "Encoding error";
-	if (s->log_timeptr) time(s->log_timeptr);
-	return EOF;
-      }
-
-      ptr=foo;
-      left=outlen;
+    if (s->saslssf != 0) {
+	/* Encode the data */  /* xxx handle left */
+	unsigned int outlen;
+	int result;
+	
+	result = sasl_encode(s->conn, (char *) ptr, left, 
+			     &encoded_output, &outlen);
+	if (result != SASL_OK) {
+	    s->error = "Encoding error";
+	    if (s->log_timeptr) time(s->log_timeptr);
+	    return EOF;
+	}
+	
+	ptr = (unsigned char *) encoded_output;
+	left = outlen;
     }
 
     /* Write out the data */
     do {
 #ifdef HAVE_SSL
-      if (s->tls_conn!=NULL)
-      {
-        n = SSL_write(s->tls_conn, ptr, left);
-      } else {
-	n = write(s->fd, ptr, left);
-      }
+	if (s->tls_conn != NULL) {
+	    n = SSL_write(s->tls_conn, (char *) ptr, left);
+	} else {
+	    n = write(s->fd, ptr, left);
+	}
 #else  /* HAVE_SSL */
 	n = write(s->fd, ptr, left);
 #endif /* HAVE_SSL */
@@ -494,9 +499,9 @@ struct protstream *s;
 	}
     } while (left);
 
-    /* sasl_encode did a malloc so we need to free for it */
-    if (s->saslssf!=0) { 
-      free(foo);
+    /* sasl_encode() did a malloc so we need to free for it */
+    if (s->saslssf != 0) { 
+	free(encoded_output);
     }
 
     /* Reset the output buffer */
@@ -510,11 +515,11 @@ struct protstream *s;
 /*
  * Write to the output stream 's' the 'len' bytes of data at 'buf'
  */
-int prot_write(s, buf, len)
-struct protstream *s;
-const char *buf;
-unsigned len;
+int prot_write(struct protstream *s, const char *buf, unsigned len)
 {
+    assert(len >= 0);
+    assert(s->write);
+
     while (len >= s->cnt) {
 	memcpy(s->ptr, buf, s->cnt);
 	s->ptr += s->cnt;
@@ -527,6 +532,8 @@ unsigned len;
     s->ptr += len;
     s->cnt -= len;
     if (s->error || s->eof) return EOF;
+
+    assert(s->cnt > 0);
     return 0;
 }
 
@@ -534,28 +541,16 @@ unsigned len;
  * Stripped-down version of printf() that works on protection streams
  * Only understands '%d', '%s', '%c', and '%%' in the format string.
  */
-#ifdef __STDC__
 int prot_printf(struct protstream *s, const char *fmt, ...)
-#else
-int prot_printf(va_alist)
-va_dcl
-#endif
 {
     va_list pvar;
     char *percent, *p;
     int i;
     unsigned u;
     char buf[30];
-#ifdef __STDC__
     va_start(pvar, fmt);
-#else
-    struct protstream *s;
-    char *fmt;
 
-    va_start(pvar);
-    s = va_arg(pvar, struct protstream *);
-    fmt = va_arg(pvar, char *);
-#endif
+    assert(s->write);
 
     while ((percent = strchr(fmt, '%')) != 0) {
 	prot_write(s, fmt, percent-fmt);
@@ -601,14 +596,12 @@ va_dcl
  * Read from the protections stream 's' up to 'size' bytes into the buffer
  * 'buf'.  Returns the number of bytes read, or 0 for some error.
  */
-int
-prot_read(s, buf, size)
-struct protstream *s;
-char *buf;
-unsigned size;
+int prot_read(struct protstream *s, char *buf, unsigned size)
 {
     int c;
 
+    assert(!s->write);
+
     if (!size) return 0;
 
     if (s->cnt) {
@@ -633,15 +626,13 @@ unsigned size;
 /*
  * Version of fgets() that works with protection streams.
  */
-char *
-prot_fgets(buf, size, s)
-char *buf;
-unsigned size;
-struct protstream *s;
+char *prot_fgets(char *buf, unsigned size, struct protstream *s)
 {
     char *p = buf;
     int c;
 
+    assert(!s->write);
+
     if (size < 2) return 0;
     size -= 2;
 
@@ -654,3 +645,44 @@ struct protstream *s;
     *p++ = '\0';
     return buf;
 }
+
+/* function versions of the macros */
+#undef prot_getc
+#undef prot_ungetc
+#undef prot_putc
+
+int prot_getc(struct protstream *s)
+{
+    assert(!s->write);
+
+    if (s->cnt-- > 0) {
+	return *(s->ptr)++;
+    } else {
+	return prot_fill(s);
+    }
+}
+
+int prot_ungetc(int c, struct protstream *s)
+{
+    assert(!s->write);
+
+    s->cnt++;
+    *--(s->ptr) = c;
+
+    return c;
+}
+
+int prot_putc(int c, struct protstream *s)
+{
+    assert(s->write);
+    assert(s->cnt > 0);
+
+    *s->ptr++ = c;
+    if (--s->cnt == 0) {
+	return prot_flush(s);
+    } else {
+	return 0;
+    }
+}
+
+
