@@ -1,24 +1,60 @@
 /* ptloader.c -- AFS group loader daemon
+ */
+/*
+ * Copyright (c) 1996-2000 Carnegie Mellon University.  All rights reserved.
  *
- * Copyright 1996-1998, Carnegie Mellon University.  All Rights Reserved.
- * 
- * This software is made available for academic and research
- * purposes only.  No commercial license is hereby granted.
- * Copying and other reproduction is authorized only for research,
- * education, and other non-commercial purposes.  No warranties,
- * either expressed or implied, are made regarding the operation,
- * use, or results of the software.  Such a release does not permit
- * use of the code for commercial purposes or benefits by anyone
- * without specific, additional permission by the owner of the code.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
 
-static char rcsid[] = "$Id: ptloader.c,v 1.15 1999/08/09 21:07:54 leg Exp $";
+static char rcsid[] __attribute__((unused)) = 
+      "$Id: ptloader.c,v 1.21 2000/05/24 05:33:24 leg Exp $";
+
+#include <config.h>
+
 #include <string.h>
 #include "auth_krb_pts.h"
 #include <stdio.h>
+#include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/param.h>
+#include <fcntl.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <sys/uio.h>
@@ -27,6 +63,12 @@ static char rcsid[] = "$Id: ptloader.c,v 1.15 1999/08/09 21:07:54 leg Exp $";
 #include <afs/auth.h>
 #include <com_err.h>
 
+#include <db.h>
+
+#include "xmalloc.h"
+#include "lock.h"
+#include "retry.h"
+
 /* blame transarc i've been told */
 #ifndef AFSCONF_CLIENTNAME
 #include <afs/dirpath.h>
@@ -37,7 +79,6 @@ static char ptclient_debug = 0;
 
 static void newclient();
 static int reauth();
-int fatal();
 
 int
 key_to_key(user,instance,realm,passwd,key)
@@ -66,7 +107,7 @@ main(argc, argv)
     extern char *optarg;
     char *user;
     char *pw_file = NULL;
-    time_t next_auth_time;
+    time_t next_auth_time = 0;
     unsigned int auth_interval;
     int do_reauth = 1, use_srvtab = 0;
     int use_newpag = 0;
@@ -76,69 +117,69 @@ main(argc, argv)
 
     /* normally LOCAL6, but do this while we're logging keys */
     openlog(PTCLIENT, LOG_PID, LOG_LOCAL7);
-    syslog(LOG_NOTICE, "starting: $Id: ptloader.c,v 1.15 1999/08/09 21:07:54 leg Exp $");
+    syslog(LOG_NOTICE, "starting: $Id: ptloader.c,v 1.21 2000/05/24 05:33:24 leg Exp $");
 
     while ((opt = getopt(argc, argv, "Uspd:l:f:u:t:")) != EOF) {
-      switch (opt) {
-      case 'U':
-	do_reauth = 0;
-	break;
-      case 's':
-	use_srvtab = 1;
-	break;
-      case 'p':
-	use_newpag = 1;
-	break;
-      case 'd':
-	ptclient_debug = atoi(optarg);
-	if (ptclient_debug < 1) {
-	  ptclient_debug = 1;
+	switch (opt) {
+	case 'U':
+	    do_reauth = 0;
+	    break;
+	case 's':
+	    use_srvtab = 1;
+	    break;
+	case 'p':
+	    use_newpag = 1;
+	    break;
+	case 'd':
+	    ptclient_debug = atoi(optarg);
+	    if (ptclient_debug < 1) {
+		ptclient_debug = 1;
+	    }
+	    break;
+	case 'l':
+	    listen_queue = atoi(optarg);
+	    break;
+	case 'f':
+	    pw_file = optarg;
+	    break;
+	case 'u':
+	    user = optarg;
+	    break;
+	case 't':
+	    auth_interval = atoi(optarg);
+	    break;
+	case '?':
+	    fprintf(stderr,"usage: -Udlfut"
+		    "\n\t-d <n>\tdebug level"
+		    "\n\t-l <n>\tlisten(2) queue backlog"
+		    "\n\t-U\tDo not reauthenticate"
+		    "\n\t-s\tAssume file is srvtab"
+		    "\n\t-u <userid>\tuser to authenticate as"
+		    "\n\t-f <file>\tfile for the users password"
+		    "\n\t-t <seconds>\tinterval between authentications"
+		    "\n\t-p\tset a new pag"
+		    "\n");
+	    syslog(LOG_ERR, "Invalid command line option specified");
+	    exit(-1);
+	    break;
+	default:
+	    break;
+	    /* just pass through */
 	}
-	break;
-      case 'l':
-	listen_queue = atoi(optarg);
-	break;
-      case 'f':
-	pw_file = optarg;
-	break;
-      case 'u':
-	user = optarg;
-	break;
-      case 't':
-	auth_interval = atoi(optarg);
-	break;
-      case '?':
-	fprintf(stderr,"usage: -Udlfut"
-		"\n\t-d <n>\tdebug level"
-		"\n\t-l <n>\tlisten(2) queue backlog"
-		"\n\t-U\tDo not reauthenticate"
-		"\n\t-s\tAssume file is srvtab"
-		"\n\t-u <userid>\tuser to authenticate as"
-		"\n\t-f <file>\tfile for the users password"
-		"\n\t-t <seconds>\tinterval between authentications"
-		"\n\t-p\tset a new pag"
-		"\n");
-	syslog(LOG_ERR, "Invalid command line option specified");
-	exit(-1);
-	break;
-      default:
-	break;
-	/* just pass through */
-      }
     }
 
     if (listen_queue < 5) {
-      if (ptclient_debug) {
-	syslog(LOG_ERR, "Invalid listen_queue specified (%d), resetting to 5",
-	       listen_queue);
-	listen_queue = 5;
-      }
+	if (ptclient_debug) {
+	    syslog(LOG_ERR, "Invalid listen_queue specified (%d), resetting to 5",
+		   listen_queue);
+	    listen_queue = 5;
+	}
     }
 
     s = socket(AF_UNIX, SOCK_STREAM, 0);
     if (s == -1) {
-      syslog(LOG_ERR, "socket: %m");
-      exit(1);
+	syslog(LOG_ERR, "socket: %m");
+	exit(1);
     }
 
     strcpy(fnamebuf, STATEDIR);
@@ -155,58 +196,58 @@ main(argc, argv)
     umask(oldumask); /* for Linux */
     chmod(fnamebuf, 0777); /* for DUX */
     if (r == -1) {
-      syslog(LOG_ERR, "bind: %s: %m", fnamebuf);
-      exit(1);
+	syslog(LOG_ERR, "bind: %s: %m", fnamebuf);
+	exit(1);
     }
     r = listen(s, listen_queue);
     if (r == -1) {
-      syslog(LOG_ERR, "listen: %m");
-      exit(1);
+	syslog(LOG_ERR, "listen: %m");
+	exit(1);
     }
 
     if (do_reauth) {
-      if (pw_file == NULL) {
-	syslog(LOG_ERR, "Invalid password file specified. Exiting...");
-	exit(-1);
-      }
-      if (reauth(user, pw_file, use_newpag, use_srvtab) < 0) {
-	syslog(LOG_ERR, "initialization failed. exiting...");
-	exit(-1);
-      }
+	if (pw_file == NULL) {
+	    syslog(LOG_ERR, "Invalid password file specified. Exiting...");
+	    exit(-1);
+	}
+	if (reauth(user, pw_file, use_newpag, use_srvtab) < 0) {
+	    syslog(LOG_ERR, "initialization failed. exiting...");
+	    exit(-1);
+	}
       
-      next_auth_time = time(0) + auth_interval;
+	next_auth_time = time(0) + auth_interval;
 
-      if (ptclient_debug > 10) {
-	syslog(LOG_DEBUG, "Authenticated as %s; next reauth at %d", 
-	       user, next_auth_time);
-      }
+	if (ptclient_debug > 10) {
+	    syslog(LOG_DEBUG, "Authenticated as %s; next reauth at %d", 
+		   user, next_auth_time);
+	}
     }
 
     len = sizeof(clientaddr);
     for (;;) {
-      c = accept(s, (struct sockaddr *)&clientaddr, &len);
-      if (c == -1) {
-	syslog(LOG_WARNING, "WARNING: accept: %m");
-	continue;
-      }
-
-      if (do_reauth && (time(0) > next_auth_time)) {
-	if (ptclient_debug > 10) {
-	  syslog(LOG_DEBUG, "Reauthenticating at %d", time(0));
+	c = accept(s, (struct sockaddr *)&clientaddr, &len);
+	if (c == -1) {
+	    syslog(LOG_WARNING, "WARNING: accept: %m");
+	    continue;
 	}
-	if (reauth(user, pw_file, use_newpag, use_srvtab) < 0) {
-	  syslog(LOG_ERR, "error reauthenticating. continuing...");
-	} else {
-	  next_auth_time = time(0) + auth_interval;
-	  if (ptclient_debug) {
-	    syslog(LOG_DEBUG, "Successfully re-authenticated.");
-	  }
-	}
-      }
 
-      newclient(c);
+	if (do_reauth && (time(0) > next_auth_time)) {
+	    if (ptclient_debug > 10) {
+		syslog(LOG_DEBUG, "Reauthenticating at %d", time(0));
+	    }
+	    if (reauth(user, pw_file, use_newpag, use_srvtab) < 0) {
+		syslog(LOG_ERR, "error reauthenticating. continuing...");
+	    } else {
+		next_auth_time = time(0) + auth_interval;
+		if (ptclient_debug) {
+		    syslog(LOG_DEBUG, "Successfully re-authenticated.");
+		}
+	    }
+	}
+
+	newclient(c);
     }
-    /*NOTREACHED*/
+    /* NOTREACHED */
 }
 
 static void
@@ -216,16 +257,17 @@ int c;
     char fnamebuf[1024];
     char keyinhex[512];
     const char *reply;
-    HASHINFO info;
     DB *ptdb;
+    DBT key, data;
     char indata[PTS_DB_KEYSIZE];
     char user[PR_MAXNAMELEN];
     namelist groups;
     int i,fd,rc;
-    DBT key,dataheader,datalist;
-    ptluser us;
-    char (*list)[][PR_MAXNAMELEN];
     size_t size;
+    struct auth_state *newstate;
+
+    memset(&key, 0, sizeof(key));
+    memset(&data, 0, sizeof(data));
 
     (void)memset(&size, 0, sizeof(size));
     if (read(c, &size, sizeof(size_t)) < 0) {
@@ -235,36 +277,37 @@ int c;
     }
 
     if (size > PTS_DB_KEYSIZE)  {
-      syslog(LOG_ERR, "size sent %d is greater than buffer size %d", 
-	     size, PTS_DB_KEYSIZE);
-      reply  = "Error: invalid request size";
-      goto sendreply;
+	syslog(LOG_ERR, "size sent %d is greater than buffer size %d", 
+	       size, PTS_DB_KEYSIZE);
+	reply = "Error: invalid request size";
+	goto sendreply;
     }
 
-    (void)memset(&indata,0,PTS_DB_KEYSIZE);
+    memset(&indata, 0, PTS_DB_KEYSIZE);
     if (read(c, &indata, size) < 0) {
         syslog(LOG_ERR,"socket (indata; size = %d): %m", size);
         reply = "Error reading request (key)";
         goto sendreply;
     }
 
-    for (i=0; i<size; i++) 
-      sprintf(keyinhex+(2*i), "%.2x", indata[i]);
+    /* convert request to hex */
+    for (i=0; i<size; i++) {
+	sprintf(keyinhex+(2*i), "%.2x", indata[i]);
+    }
 
-    (void)memset(&user, 0, sizeof(user));
+    memset(&user, 0, sizeof(user));
     if (read(c, &user, PR_MAXNAMELEN) < 0) {
-        syslog(LOG_ERR, "socket(user; size = %d; key = %s): %m", size, keyinhex);
+        syslog(LOG_ERR, "socket(user; size = %d; key = %s): %m", 
+	       size, keyinhex);
         reply = "Error reading request (user)";
         goto sendreply;
     }      
 
     if (ptclient_debug) {
-      syslog(LOG_DEBUG, "user %s, cacheid %s", user, keyinhex);
+	syslog(LOG_DEBUG, "user %s, cacheid %s", user, keyinhex);
     }
 
-    (void)memset(&info, 0, sizeof(info));
-
-    (void)memset(&groups, 0, sizeof(groups));
+    memset(&groups, 0, sizeof(groups));
     groups.namelist_len = 0;
     groups.namelist_val = NULL;
     
@@ -274,100 +317,79 @@ int c;
         goto sendreply;
     }
 
-    (void)memset(&key, 0, sizeof(key));
     key.data = indata;
     key.size = size;
 
-    us.ngroups = groups.namelist_len;
-    us.cached = time(0);
+    /* fill in our new state structure */
+    data.size = sizeof(struct auth_state) + 
+	(groups.namelist_len * PR_MAXNAMELEN);
+    newstate = (struct auth_state *) xmalloc(data.size);
+    data.data = newstate;
+    strcpy(newstate->userid, user);
+    kname_parse(newstate->aname, newstate->inst, newstate->realm, user);
+    newstate->mark = time(0);
+    newstate->ngroups = groups.namelist_len;
     /* store group list in contiguous array for easy storage in the database */
-    list = (char (*)[][PR_MAXNAMELEN])xmalloc(us.ngroups*PR_MAXNAMELEN);
-    memset(list, 0, us.ngroups * PR_MAXNAMELEN);
-    for (i=0; i<us.ngroups; i++){
-        strcpy((*list)[i], groups.namelist_val[i]);
+    memset(newstate->groups, 0, newstate->ngroups * PR_MAXNAMELEN);
+    for (i = 0; i < newstate->ngroups; i++) {
+        strcpy(newstate->groups[i], groups.namelist_val[i]);
 	/* don't free groups.namelist_val[i]. Something else currently
 	 * takes care of that data. 
 	 */
     }
     if (groups.namelist_val != NULL) {
-      free(groups.namelist_val);
+	free(groups.namelist_val);
     }
 
-    /* build and store a header record for this user */
-    strcpy(us.user, user);
-    
-    (void)memset(&dataheader, 0, sizeof(dataheader));
-    dataheader.data = &us;
-    dataheader.size = sizeof(ptluser);
-    datalist.data = list;
-    datalist.size = us.ngroups*PR_MAXNAMELEN;
-    indata[PTS_DB_HOFFSET] = 'H';
-
+    /* lock database */
     strcpy(fnamebuf, STATEDIR);
     strcat(fnamebuf, PTS_DBLOCK);
     fd=open(fnamebuf, O_CREAT|O_TRUNC|O_RDWR, 0664);
     if (fd == -1) {
         syslog(LOG_ERR, "IOERROR: creating lock file %s: %m", fnamebuf);
         reply="Couldn't create lock file";
-	free(list);
+	free(newstate);
         goto sendreply;
     }
     if (lock_blocking(fd) < 0) {
         syslog(LOG_ERR, "IOERROR: locking lock file %s: %m", fnamebuf);
-        reply="Couldn't lock database";
-	free(list);
+        reply ="Couldn't lock database";
+	free(newstate);
         goto sendreply;
     }
+
+    /* store to database */
     strcpy(fnamebuf, STATEDIR);
     strcat(fnamebuf, PTS_DBFIL);
-    ptdb=dbopen(fnamebuf, O_RDWR, 0, DB_HASH, &info);
-    if (ptdb == NULL) {
-        syslog(LOG_ERR, "IOERROR: opening database %s: %m", fnamebuf);
+    rc = db_create(&ptdb, NULL, 0);
+    if (!rc) rc = ptdb->open(ptdb, fnamebuf, NULL, DB_HASH, DB_CREATE, 0664);
+    rc = ptdb->open(ptdb, fnamebuf, NULL, DB_HASH, 0, 0664);
+    if (rc != 0) {
+        syslog(LOG_ERR, "IOERROR: opening database %s: %s", fnamebuf,
+	       db_strerror(rc));
         close(fd);
         reply="Couldn't open database";
-	free(list);
-        goto sendreply;
-    }
-    if (ptclient_debug > 10) {
-      for (i=0; i<size; i++) 
-	sprintf(keyinhex+(2*i), "%.2x", indata[i]);
-      syslog(LOG_DEBUG, "user %s: header key: %s", user, keyinhex);
-    }
-
-    rc=PUT(ptdb, &key, &dataheader, 0);
-    if (rc < 0) {
-        syslog(LOG_ERR, "IOERROR: writing header into database %s: %m", fnamebuf);
-        CLOSE(ptdb);
-        close(fd);
-        reply="Couldn't write database";
-	free(list);
+	free(newstate);
         goto sendreply;
     }
 
-    /* store the grouplist */
-    indata[PTS_DB_HOFFSET] = 'D';
-    if (ptclient_debug > 10) {
-      for (i=0; i<size; i++) 
-	sprintf(keyinhex+(2*i), "%.2x", indata[i]);
-      syslog(LOG_DEBUG, "user %s: grouplist key: %s", user, keyinhex);
-    }
-
-    rc=PUT(ptdb, &key, &datalist, 0);
-    if (rc < 0) {
-        syslog(LOG_ERR, "IOERROR: writing data into database %s: %m", fnamebuf);
-	/* We don't do any cleanup as this is the last function in line. 
-	 * if we actually exit here, don't forget to clean up. */
-    }
-    CLOSE(ptdb);
+    rc = ptdb->put(ptdb, NULL, &key, &data, 0);
+    free(newstate);
+    ptdb->close(ptdb, 0);
     close(fd);
+    if (rc != 0) {
+        syslog(LOG_ERR, "IOERROR: writing header into database %s: %s", 
+	       fnamebuf, db_strerror(rc));
+        reply = "Couldn't write database";
+        goto sendreply;
+    }
 
     /* and we're done */
-    free(list);
-    reply="OK";
+    reply = "OK";
     
-sendreply:
+ sendreply:
     if (retry_write(c, reply, strlen(reply)) <0) {
-      syslog(LOG_WARNING, "retry_write: %m");
+	syslog(LOG_WARNING, "retry_write: %m");
     }
     close(c);
 }
@@ -409,7 +431,7 @@ reauth(name, file, newpag, is_srvtab)
     CREDENTIALS c;
     struct ktc_principal aserver;
     struct ktc_principal aclient;
-    struct ktc_token atoken, btoken;
+    struct ktc_token atoken;
 
 
     if (newpag)
@@ -452,8 +474,8 @@ reauth(name, file, newpag, is_srvtab)
       char *t = aserver.cell;
       char *s = aserver.cell;
       int c;
-      while (c = *t++) {
-        if (isupper(c)) c=tolower(c);
+      while ((c = *t++)) {
+        if (isupper(c)) c = tolower(c);
         *s++ = c;
       }
       *s++ = 0;
@@ -519,12 +541,9 @@ reauth(name, file, newpag, is_srvtab)
 /* we need to have this function here 'cause libcyrus.a 
  * makes calls to this function. 
  */
-int
-fatal(msg, exitcode)
-char *msg;
-int exitcode;
+void fatal(const char *msg, int exitcode)
 {
-  syslog(LOG_ERR, "%s", msg);
-  exit(-1);
+    syslog(LOG_ERR, "%s", msg);
+    exit(-1);
 }
-/* $Header: /afs/andrew.cmu.edu/system/cvs/src/cyrus/ptclient/ptloader.c,v 1.15 1999/08/09 21:07:54 leg Exp $ */
+/* $Header: /afs/andrew.cmu.edu/system/cvs/src/cyrus/ptclient/ptloader.c,v 1.21 2000/05/24 05:33:24 leg Exp $ */
