@@ -2,29 +2,48 @@
  * Tim Martin
  * 9/21/99
  */
-/***********************************************************
-        Copyright 1999 by Carnegie Mellon University
-
-                      All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of Carnegie Mellon
-University not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
-ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-******************************************************************/
+/*
+ * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ */
 
+#include <config.h>
 
 #include <sasl.h> /* yay! sasl */
 
@@ -48,7 +67,7 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <string.h>
 
 #include "prot.h"
-#include "config.h"
+#include "imapconf.h"
 #include "xmalloc.h"
 
 #include "codes.h"
@@ -77,22 +96,9 @@ int sieved_userisadmin;
 
 void cmdloop()
 {
-  unsigned int mechcount;
-  char *sasllist;
-
   chdir("/tmp/");
 
-  prot_printf(sieved_out, "\"" SIEVED_IDENT " " SIEVED_VERSION "\"");
-
-  if (sasl_listmech(sieved_saslconn, NULL, 
-		    " \"SASL={", ", ", "}\"",
-		    &sasllist,
-		    NULL, &mechcount) == SASL_OK && mechcount > 0)
-    {
-      prot_printf(sieved_out,"%s",sasllist);
-    }
-    
-  prot_printf(sieved_out,"\r\n");
+  capabilities(sieved_out, sieved_saslconn);
 
   /* initialize lexer */
   lex_init();
@@ -120,96 +126,13 @@ void fatal(const char *s, int code)
 
 }
 
-/* This creates a structure that defines the allowable
- *   security properties 
- */
-static sasl_security_properties_t *make_secprops(int min,int max)
-{
-  sasl_security_properties_t *ret=
-    (sasl_security_properties_t *) xmalloc(sizeof(sasl_security_properties_t));
-
-  ret->maxbufsize = 4000;
-  ret->min_ssf = min;		/* minimum allowable security strength */
-  ret->max_ssf = max;		/* maximum allowable security strength */
-
-  ret->security_flags = 0;
-  if (!config_getswitch("allowplaintext", 1)) {
-      ret->security_flags |= SASL_SEC_NOPLAINTEXT;
-  }
-
-  /* never allow anonymous */
-  ret->security_flags |= SASL_SEC_NOANONYMOUS;
-
-  ret->property_names = NULL;
-  ret->property_values = NULL;
-
-  return ret;
-}
-
-/* this is a wrapper to call the cyrus configuration from SASL */
-static int mysasl_config(void *context, 
-			 const char *plugin_name,
-			 const char *option,
-			 const char **result,
-			 unsigned *len)
-{
-    char opt[1024];
-
-    if (strcmp(option, "srvtab")) { /* we don't transform srvtab! */
-	int sl = 5 + (plugin_name ? strlen(plugin_name) + 1 : 0);
-
-	strncpy(opt, "sasl_", 1024);
-	if (plugin_name) {
-	    strncat(opt, plugin_name, 1019);
-	    strncat(opt, "_", 1024 - sl);
-	}
- 	strncat(opt, option, 1024 - sl - 1);
-	opt[1023] = '\0';
-    } else {
-	strncpy(opt, option, 1024);
-    }
-
-    *result = (const char *) config_getstring(opt, NULL);
-    if (*result != NULL) {
-	if (len) { *len = strlen(*result); }
-	return SASL_OK;
-    }
-   
-    return SASL_FAIL;
-}
-
-/* returns true if imapd_authstate is in "item";
-   expected: item = admins or proxyservers 
-*/
-static int authisa(const char *item)
-{
-    const char *val = config_getstring(item, "");
-    char buf[1024];
-
-    while (*val) {
-	char *p;
-	
-	for (p = (char *) val; *p && !isspace(*p); p++);
-	strncpy(buf, val, p-val);
-	buf[p-val] = 0;
-
-	if (auth_memberof(sieved_authstate, buf)) {
-	    return 1;
-	}
-	val = p;
-	while (*val && isspace(*val)) val++;
-    }
-    return 0;
-}
-
-
 /* should we allow users to proxy?  return SASL_OK if yes,
    SASL_BADAUTH otherwise */
 static int mysasl_authproc(void *context,
-		       const char *auth_identity,
-		       const char *requested_user,
-		       const char **user,
-		       const char **errstr)
+			   const char *auth_identity,
+			   const char *requested_user,
+			   const char **user,
+			   const char **errstr)
 {
     const char *val;
     char *canon_authuser, *canon_requser;
@@ -254,14 +177,24 @@ static int mysasl_authproc(void *context,
     sieved_authstate = auth_newstate(canon_authuser, NULL);
 
     /* ok, is auth_identity an admin? */
-    sieved_userisadmin = authisa("admins");
+    sieved_userisadmin = authisa(sieved_authstate, "sieve", "admins");
 
-    if (strcmp(canon_authuser, canon_requser) != 0) {
-	if (sieved_userisadmin || authisa("proxyservers")) {
-	    /* proxy ok! */
-	    sieved_userisadmin = 0;
+    /* we want to authenticate as a different user: ok if we're an admin or
+     a proxy server */
+    if (strcmp(canon_authuser, canon_requser)) {
+	if (sieved_userisadmin && authisa(sieved_authstate, "sieve", 
+					  "proxyservers")) {
+	    sieved_userisadmin = 0; /* no longer admin */
+	    auth_freestate(sieved_authstate);
+	    
+	    sieved_authstate = auth_newstate(canon_requser, NULL);
 	} else {
-	    /* not authorized to proxy */
+	    *errstr = "user is not allowed to proxy";
+	    
+	    free(canon_authuser);
+	    free(canon_requser);
+	    auth_freestate(sieved_authstate);
+
 	    return SASL_BADAUTH;
 	}
     }
@@ -278,22 +211,25 @@ static struct sasl_callback mysasl_cb[] = {
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
-int main(int argc, char **argv, char **envp)
+int service_init(int argc, char **argv, char **envp)
+{
+
+
+    return 0;
+}
+
+int service_main(int argc, char **argv, char **envp)
 {
     int salen;
     struct hostent *hp;
     int timeout;
-    char hostname[MAXHOSTNAMELEN+1];
     sasl_security_properties_t *secprops = NULL;
 
-    if (gethostname(hostname, MAXHOSTNAMELEN)!=0)
-      fatal("gethostname failed\n",-5);
-
     /* set up the prot streams */
     sieved_in = prot_new(0, 0);
     sieved_out = prot_new(1, 1);
 
-    config_init("timsieved");
+    config_changeident("timsieved");
     timeout = config_getint("timeout", 10);
     if (timeout < 10) timeout = 10;
     prot_settimeout(sieved_in, timeout * 60);
@@ -335,14 +271,14 @@ int main(int argc, char **argv, char **envp)
 	fatal("SASL failed initializing: sasl_server_init()", -1); 
 
     /* other params should be filled in */
-    if (sasl_server_new("imap", hostname, NULL, NULL, SASL_SECURITY_LAYER, 
-			&sieved_saslconn)
+    if (sasl_server_new("imap", NULL, NULL, 
+			NULL, SASL_SECURITY_LAYER, &sieved_saslconn)
 	   != SASL_OK)
 	fatal("SASL failed initializing: sasl_server_new()", -1); 
 
     /* will always return something valid */
     /* should be configurable! */
-    secprops = make_secprops(0, 2000);
+    secprops = mysasl_secprops(0);
     sasl_setprop(sieved_saslconn, SASL_SEC_PROPS, secprops);
     
     sasl_setprop(sieved_saslconn, SASL_IP_REMOTE, &sieved_remoteaddr);  
@@ -354,7 +290,7 @@ int main(int argc, char **argv, char **envp)
     cmdloop();
 
     /* never reaches */
-    return -99;
+    exit(-99);
 }
 
 
